///////////////
// SideBar.h //
///////////////

#ifndef SIDEBAR_H
#define SIDEBAR_H

// Includes
#include <stdio.h>		// Includes define of "NULL"

//#define OEMRESOURCE
#define STRICT
#include <windows.h>

#include <string.h>		// Includes define of "NULL"
#include <math.h>

#include <time.h>
#include <sys/types.h>
#include <sys/timeb.h>

// BOOL, TRUE, and FALSE are defined in "windef.h",
// which is included via "windows.h".
#define FAIL -1					/* Function return	*/
#define SUCCEED 0				/* Function return	*/

#include "datetime.h"
#include "lnklst.h"

// Defines
#define APPNAME				"SideBar, V3.0.T0, 02 May 2003, USGS"
#define WNDCLASSNAME		"C_SideBar"

#define LOGO_ID				(LPCTSTR)"IDB_USGS_LOGO"
#define LOGO_IDint			((int)101)
#define LOGO_FILENAME		"USGSid_color_666mil.bmp"
#define LOGO_WIDTH			((int)206)
#define LOGO_WIDTH_OFF		((int)3)
#define LOGO_HEIGHT			((int)78)
#define LOGO_TOPGTTR		((int)3)

// Custom Windows messages
#define WM_UPDATE_WIN		(WM_USER + 1101)

// IDs for WM_COMMAND messages (unique IDs for various controls)

// Misc IDs
#define IDM_BASE			(100)
#define IDM_LOGOS			(IDM_BASE + 1)

// Button IDs
#define IDB_BASE			(200)
#define IDB_OLDEST			(IDB_BASE + 1)
#define IDB_MOST_RECENT		(IDB_BASE + 2)
#define IDB_PREVIOUS		(IDB_BASE + 3)
#define IDB_PREVIOUS_X5		(IDB_BASE + 4)
#define IDB_BIGGEST			(IDB_BASE + 5)
#define IDB_NEXT			(IDB_BASE + 6)
#define IDB_NEXT_X5			(IDB_BASE + 7)

#define IDB_QUIT			(IDB_BASE + 8)

#define IDB_PGA_SELECT		(IDB_BASE + 9)
#define IDB_PGV_SELECT_IN	(IDB_BASE + 10)
#define IDB_PGV_SELECT_CM	(IDB_BASE + 11)


#define INCHES_PER_S		((int)1)
#define CM_PER_S			((int)2)

// Edit-window IDs
#define IDE_BASE			(400)
#define IDE_PREMETRIC		(IDE_BASE + 1)
#define IDE_METRIC			(IDE_BASE + 2)
#define IDE_DATE_TIME		(IDE_BASE + 3)
#define IDE_WHICH_OF		(IDE_BASE + 4)
#define IDE_SITE_INFO		(IDE_BASE + 5)
#define IDE_MSGWARN			(IDE_BASE + 6)

// Misc magic numbers
#define GUTTER_SIZE			((int)4)	// Width or height of gutters.
#define HALF_GUTTER			((int)2)	// Width or height of gutters.
#define DEF_WIDTH			((int)300)	// Default width of main window.
#define DEF_HEIGHT			((int)400)	// Default height of main window.
#define CONTROL_WIDTH		(GUTTER_SIZE + LOGO_WIDTH + GUTTER_SIZE)
										// Width reserved for controls
										//   (INcluding gutters).
#define BUTTON_WIDTH		((int)75)	// Width of buttons.
#define SCALE_WIDTH			((int)55)	// (Fixed) width of Intensity scale ...
#define LABLE_WIDTH			((int)80)	// ... and each of its two lable bars.

// CAVEAT:  The following two really should be obtained from the system.
#define CAPTIONH			((int)19)	// Windows caption-bar height
#define WNDBDR				((int)4)	// Windows border width

#define COM1_BUFSIZE		((int)64000)// Read-buffer size
#define BUF_SIZE			((int)512)	// General character buffer size
#define SHORT_STR			((int)16)
#define MEDIUM_STR			((int)24)
#define LONG_STR			((int)32)

#define WARN_BUF_SIZE		((int)1024)

#define MAX_RAVEN_UDP		((int)1028)
					// Biggest UDP packet a Raven modem can send

#define NULL_WAVE_P			((short int)(32767))
#define NULL_WAVE_M			((short int)(-32767))
					// Waveform magic-marker numbers

#define SHOW_ANCIENT		((int)1)	// Display oldest available earthquake.
#define SHOW_RECENT			((int)2)	// Display most recent earthquake.
#define SHOW_BIGGEST		((int)3)	// Display biggest earthquake.
#define SHOW_SELECTED_P		((int)4)	// Display specific, selected earthquake (previous).
#define SHOW_SELECTED_N		((int)5)	// Display specific, selected earthquake (next).


/////////////////////////////////
// ShakeMap color scale (SMCS[]):
// (interpolate linearly beween)
//
// MMI         R       G       B
// ------------------------------
//   0        255     255     255
//   1        255     255     255			// Lowest MMI from PGAVtoMMI()
//   2        191     204     255
//   3        160     230     255
//   4        128     255     255
//   5        122     255     147
//   6        255     255       0
//   7        255     200       0
//   8        255     145       0
//   9        255       0       0
//  10        200       0       0			// Highest MMI from PGAVtoMMI()
//  11        176       0       0			// Added by JRE
//  12        152       0       0			// Added by JRE
//  13        128       0       0
/////////////////////////////////

#define N_COLOR_SLICES			((int)100)	// Number of color slices into which the ShakeMap
											// color scale is divided for presentation here.

#define MMI_PHRASES				((int)24)	// Maximum length of the following phrases
// ShakeMap "Perceived Shaking"
#define SM_SHAKE_PERCVD_I		"Rarely Felt"
#define SM_SHAKE_PERCVD_II_III	"Weak"
#define SM_SHAKE_PERCVD_IV		"Light"
#define SM_SHAKE_PERCVD_V		"Moderate"
#define SM_SHAKE_PERCVD_VI		"Strong"
#define SM_SHAKE_PERCVD_VII		"Very Strong"
#define SM_SHAKE_PERCVD_VIII	"Severe"
#define SM_SHAKE_PERCVD_IX		"Violent"
#define SM_SHAKE_PERCVD_X		"Extreme"

// ShakeMap "Potential Damage"
#define SM_POT_DMG_I			"(none)"
#define SM_POT_DMG_II_III		"(none)"
#define SM_POT_DMG_IV			"(none)"
#define SM_POT_DMG_V			"Very Light"
#define SM_POT_DMG_VI			"Light"
#define SM_POT_DMG_VII			"Moderate"
#define SM_POT_DMG_VIII			"Moderate/Heavy"
#define SM_POT_DMG_IX			"Heavy"
#define SM_POT_DMG_X			"Very Heavy"

#define THRESH_MMI_SAVE		((double)6.0)
#define N2PROTECT			((int)3)

#define THRESH_MMI_CLIP		((double)10.0)


#define WHITE				RGB((BYTE)255, (BYTE)255, (BYTE)255)
//#define RED				RGB((BYTE)255, (BYTE)0,   (BYTE)0  )
//#define ORANGE			RGB((BYTE)255, (BYTE)160, (BYTE)0  )
//#define YELLOW			RGB((BYTE)255, (BYTE)255, (BYTE)0  )
//#define GREEN				RGB((BYTE)0,   (BYTE)255, (BYTE)0  )
//#define BLUE				RGB((BYTE)0,   (BYTE)0,   (BYTE)255)
#define BLACK				RGB((BYTE)0,   (BYTE)0,   (BYTE)0  )

#define VERTICAL			((int)1)
#define NORTH				((int)2)
#define EAST				((int)3)
#define TEMPERATURE			((int)4)

// Timers
#define TIMER_CLEAN			((int)1)				// ID for cleanup timer
#define CLEANUP_CHECK		((unsigned int)64007)	// ms
#define TIMER_COM1			((int)2)				// ID for COM1: checks
#define COM1_CHECK			((unsigned int)53)		// ms

// Numerical constants
#define INtoCM				((double)2.54)
#define ADCtoV				((double)10. / (double)4095.)
					// * to convert ADC counts to Volts
#define ONE_G				((double)980.665)
					// gal, cm/s^2, mean surface gravity
#define ZIP					((double)0.0)
#define HALF				((double)0.5)
#define ONE					((double)1.0)
#define TWO					((double)2.0)
#define FIVE				((double)5.0)
#define SEVEN				((double)7.0)
#define TEN					((double)10.0)
#define SIXTEEN				((double)16.0)
#define HH					((double)100.)
#define KK					((double)1000.)
#define INF					HUGE_VAL				// "Infinity"
#define PI					((double)3.14159265358979324)
#define TWOPI				(TWO * PI)


// Thermistor constants
#define R0					((double)49800.0)		// Ohms in series with thermistor
#define Vref				((double)2.50)			// Reference voltage across series pair
#define AAA					((double)1.034034113e-03)// Coefficients of cubic
#define BBB					((double)2.383800240e-04)
#define DDD					((double)1.592455177e-07)// CCC=0

#define VerboseLB			"\r\n          "		// Used by VerboseTRG() for line breaks

// Define known Sa periods and corresponding annotations //////////////////
// (MUST correspond to what ft() sends.)
#define SaPer1				( 300)		// ms
#define SaPer2				(1000)		// ms
#define SaPer3				(3000)		// ms

#define SaNote1				"Sa#1"
#define SaNote2				"Sa#2"
#define SaNote3				"Sa#3"

// Macros
#define MIN(a, b)			((a) < (b) ? (a) : (b))
//#define MIN3(a, b, c)		(MIN(MIN(a, b), c))
//#define MIN4(a, b, c, d)	(MIN(MIN(a, b), MIN(c, d)))
#define MAX(a, b)			((a) > (b) ? (a) : (b))
#define MAX3(a, b, c)		(MAX(MAX(a, b), c))
#define RANGE(mn, a, mx)	(MIN(mx, MAX(mn, a)))
#define streq				!strcmp
#define strneq				!strncmp

// Errors returned
#define TSERR_NoWndClassRegd	1
#define TSERR_NoWndCreated		2

// Message to calling routine is "back out gracefully"
#define VERBOSE_ERR(nt)	{if (nt != ntoks) \
							{return(NULL);} \
						}

///////////////////////////////////////////////////////////////////////////
// Structure typedefs

// Main data structure used by SideBar() to store site-specific information
typedef struct {
	char			TREMOR_SN[MEDIUM_STR];	// TREMOR unit at site

	double			latitude;				// Site location ...
	double			longitude;
	double			elevation;
	double			N_azimuth;				// ... and orientation of "North" ...
	double			N_azm_err;				// ... and its uncertainty (degrees) ...
	char			StationSite[LONG_STR];	// ... and site name/description.
	int				MaxDBaseLen;			// Maximum event database size.
	double			MemoryDuration;			// SideBar memory duration (s).
	double			TimeFudge;				// Approximate lag from earthquake
											// onset to "TRG0" message (s).


	char			V_ICS_SN[SHORT_STR];	// Sensor S/N (ICS)
	double			V_Sensitivity;			// Sensor sensitivity ("gain") at
	char			N_ICS_SN[SHORT_STR];	//    .                ~22.5 C (V/gal)
	double			N_Sensitivity;			//    .
	char			E_ICS_SN[SHORT_STR];	//    .
	double			E_Sensitivity;			//    .
											// (Reference temperature should be
											// entered via cal files, and a
											// variable here.  22.5 C is nominal.)

	double			V_TCS, V_Tref;			// Gain temperature sensitivities
	double			N_TCS, N_Tref;			// (fraction/C) and respective
	double			E_TCS, E_Tref;			// reference temperatures (C).

	int				LastPktSize;			// Used to identify redundant packets
	int				LastPktID;				// ==0 if "none" or "unknown", so ft()
											// must NEVER issue a packet number 0.

	char			chPktArrTime[LONG_STR];	// Ascii version of:
	double			PktArrTime;				// System time that the most
											// recent "TRG0" arrived.

	char			Warn[WARN_BUF_SIZE];	// Text buffer for warnings and
											// messages.
	int				WarnSize;				// Number of characters in it now
											// (less terminating null).

} SiteData;

// Structure used by VerboseTRG() so stations won't trip
// over one another during translation, but will remember
// certain information from one UDP packet to another or
// even from one line to another.
typedef struct {
	double	sps;					// Sample rate
	BOOL	sps_well_known;			// TRUE when known from a BGN0, FALSE when assumed.
									// (CAVEAT:  set but not used -- SHOULD be used)
	BOOL	EqTimesOK;				// Whether PreEq and TotalEq are real.
	double	PreEq;					// Pre-trigger duration (s)
	double	TotalEq;				// Total record duration (s)
									//--------------------------------------
	double	TmprtrA;				// C (for use with Arias, PSV, PGV, and Sa)
	double	TmprtrW;				// C (used ONLY to correct waveforms)
									//--------------------------------------
	int		WaveCorrQual;			// 0 = uncorrected; 1 = fair correction;
									// 2 = corrected identically to trigger.
									//--------------------------------------
	double	soh_time;				// Corrected SOH arrival times.
	double	RawTriggerTime;
	double	TrueTriggerTime;
									//--------------------------------------
	double	wave_time;
	int		WaveLinesRemaining;		// Used by VerboseTRG() to monitor the
									// progress toward the end of a waveform
									// UDP packet.  >0 causes translation of a
									// line which otherwise would be passed as
									// is because it would be "unrecognized".
	int		WaveLinesCh;			// Channel number of the UDP packet
	unsigned int	err_flgs;		// Computational-error flags from ft()
} VerSave;

// Earthquake-database entry
typedef struct {
	double			Time;					// Epochal time this earthquake was received.
	char			chTime[LONG_STR];		// Ascii version of same time.
	double			PGAch1;					// Vertical-channel   PGA (%g).
	double			PGAch2;					// Horizontal-channel PGA (%g).
	double			PGAch3;					// Horizontal-channel PGA (%g).
	double			PGVch1;					// Vertical-channel   PGV (cm/s).
	double			PGVch2;					// Horizontal-channel PGV (cm/s).
	double			PGVch3;					// Horizontal-channel PGV (cm/s).
	double			MMI;					// Instrumental Intensity
	BOOL			LargestMMIs;			// TRUE if one of the N2PROTECT largest intensities;
	double			Temperature;			// Temperature at earthquake time.
} EqEntry;


// Windows-layout structure
typedef struct {
	LONG		WinSizeX;		// Main Window size, X
	LONG		WinSizeY;		// Main Window size, Y
	LONG		ViewSizeX;		// Main Viewport size, X
	LONG		ViewSizeY;		// Main Viewport size, Y
	LONG		WndBdr;			// Border width ((WinSizeX - ViewSizeX) / 2)
	LONG		CaptionH;		// Caption bar height (WinSizeY - ViewSizeY - (2 * WndBdr))
} WinLayout;


///////////////////////////////////////////////////////////////////////////
// Function prototypes.  This is their order in file "SideBar.c", excepting
// those indicated in other source files.  They are alphabetical except for
// WndProc()

LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
// then alphabetic
BOOL		CheckMMImatches(double, double *);
void		CleanMemory(double);
double		Compensate(int, double, short int);
double		CompPGV(long, double, double, double, double, double);
double		CompSa(long, double, double, double, double, double);
void		ConfigReader();
void		Controls(HWND, int, int);
EqEntry		*CreateEqItem(double, char *,
				double, double, double,
				double, double, double,
				double);
void		DeleteEqItem(char *);
BOOL		DeleteSmallestEq();
BOOL		DrawMMIBar(HDC);
void		DrawBitmap(HDC, HBITMAP, int, int);	// in DrawBitmap.c
void		DrawScalesLables(HDC, double);
char *		EpochToStr(double);
void		ErrExit(char *);
void		ErrWarn(char *);
HFONT		EzCreateFont(HDC, char *, int, int, int, BOOL);
char *		FIFOgets(int *, char *);
BOOL		FIFOstuff(int *, int, char *, char *, int);
void		FindLargestEvents();
void		GetCurrentEq();
void		GetWindowLayout(HWND, WinLayout *);
void		GrabTrg(char *);
void		HighlightEventButton();
void		HighlightMeasureButton();
int			ListPosition();
void		MakeControls(HINSTANCE);
Bool		MatchEvent(char *, char *);
BOOL		MatchSize();
void		MMIphrases(double, char *, char *);
void		ObjectLayout();
double		PGAVtoMMI(double, double);
void		ShakeMapColor(int *, int *, int *, double);
HANDLE		StartCom1();
double		Temperature(short int);
double		VBat(short int);
char *		VerboseTRG(char *);
BOOL		WriteDataBase();

#endif // SIDEBAR_H
