/////////////////////////
// SideBar_implement.c //
/////////////////////////

// Implementation process (and its subroutines) for SideBar

#include "SideBar.h"

// Global variables
	HWND			ghWndMain;				// Globally accessable version of hWndMain
	WinLayout		WMSize;					// Size and layout of main window.

	HANDLE			Com1;					// COM1 handle.
	char			Com1Buff[COM1_BUFSIZE];	// Input buffer for COM1 ...
	DWORD			nBytesRead;				// ... and number of bytes in it.
	char			Com1FIFO[COM1_BUFSIZE];	// FIFO for COM1 ...
	int				nCom1FIFO;				// ... and number of data in it.
	char			*FIFOline;				// Oldest line of text from FIFO.
	int				PGVscale;				// Signals whether PGV is in in/s or cm/s

	SiteData		Site;					// Main data structure for SideBar()

	LONG			OriginX, OriginY;		// Upper left corner of window.
	LONG			MainHeight;				// Height of window.
	LONG			MainWidth;				// Width of window.

	LONG			CAWidth;				// Client-area dimensions.
	LONG			CAHeight;

	HBITMAP			hLogo;					// Handle to Logo bitmap
	LONG			LogoX, LogoY;			// Position of Logo area ...
	LONG			LogoH, LogoW;			// ... and its size.

	LONG			MetricEtcX, MetricEtcY;	// Text window for MMI, perceived shaking, and potential damage ...
	LONG			MetricEtcH, MetricEtcW;	// ... and its size.

	LONG			MaximaX, MaximaY;		// Text for PGA and PGV ...
	LONG			MaximaH, MaximaW;		// ... and its size.

	LONG			DateTimeX, DateTimeY;	// Text for date and time of event ...
	LONG			DateTimeH, DateTimeW;	// ... and its size.

	LONG			WhichOfX, WhichOfY;		// Text for earthquake id and count ...
	LONG			WhichOfH, WhichOfW;		// ... and its size.

	LONG			OldestX, OldestY;		// "Oldest" button ...
	LONG			OldestH, OldestW;		// ... and its size.

	LONG			MostRecentX, MostRecentY;// "Most Recent" button ...
	LONG			MostRecentH, MostRecentW;// ... and its size.

	LONG			BiggestX, BiggestY;		// "Biggest" button ...
	LONG			BiggestH, BiggestW;		// ... and its size.

	LONG			PrevX, PrevY;			// "Previous" button ...
	LONG			PrevH, PrevW;			// ... and its size.

	LONG			PrevX5X, PrevX5Y;		// "Skip back five events" button ...
	LONG			PrevX5H, PrevX5W;		// ... and its size.

	LONG			NextX, NextY;			// "Next" button ...
	LONG			NextH, NextW;			// ... and its size.

	LONG			NextX5X, NextX5Y;		// "Skip forward five events" button ...
	LONG			NextX5H, NextX5W;		// ... and its size.

	LONG			SiteInfoX, SiteInfoY;	// Site information area ...
	LONG			SiteInfoH, SiteInfoW;	// ... and its size.

	LONG			QuitX, QuitY;			// "Quit" button ...
	LONG			QuitW, QuitH;			// ... and its size.

	LONG			MsgWarnX, MsgWarnY;		// Messages/warnings window ...
	LONG			MsgWarnW, MsgWarnH;		// ... and its size.

	LONG			PGVslctInX, PGVslctInY;	// Button to display PGV values (in/s) ...
	LONG			PGVslctInH, PGVslctInW;	// ... and its size.

	LONG			PGVslctCmX, PGVslctCmY;	// Button to display PGV values (cm/s) ...
	LONG			PGVslctCmH, PGVslctCmW;	// ... and its size.
	LONG			PGVcmButtomWidth;		// (Text) width needed for "(cm/s)"

	LONG			IntTtlX, IntTtlY;		// Location of titles for bargraph, etc. ...
	LONG			IntTtlH, IntTtlW;		// ... and its size.

	LONG			BarX, BarY;				// Location of bargraph ...
	LONG			BarH, BarW;				// ... and its size.
	LONG			RqstdBarH, RqstdBarW;	// ... and its requested size, per the user.

	LONG			ScaleX, ScaleY;			// Location of bargraph scale (width
	LONG			ScaleH, ScaleW;			// is SCALE_WIDTH; height is BarH).
	LONG			ScaleWidth;				// Text sensitive version of SCALE_WIDTH
											// (Keyed to 18 pt bold, which on the development
											// machine had a width of 9 pixels.);

	LONG			PrcvdShkX, PrcvdShkY;	// Location of scale label A (perceived shaking) ...
	LONG			PrcvdShkH, PrcvdShkW;	// ... and its size.
	LONG			LableWidth;				// Text sensitive version of LABLE_WIDTH;

	LONG			PotDmgX, PotDmgY;		// Location of scale label B (potential damage) ...
	LONG			PotDmgH, PotDmgW;		// ... and its size.

	LONG			ControlX, ControlY;		// Position of Control area ...
	LONG			ControlH, ControlW;		// ... and its size.

	HWND			hWndLogo;				// Handle to logo display
	HWND			hWndMetricEtc;			// Handle to MMI/perceived-shaking/potential-damage display
	HWND			hWndMaxima;				// Handle to PGA/PGV display.
	HWND			hWndDateTime;			// Handle to date-time display.
	HWND			hWndWhichOf;			// Handle to earthquake ID-and-count display.
	HWND			hWndSiteInfo;			// Button to site-info display.
	HWND			hWndOldest;				// Handle to "Oldest" button.
	HWND			hWndMostRecent;			// Handle to "Most Recent" button.
	HWND			hWndBiggest;			// Handle to "Biggest" button.
	HWND			hWndPrev;				// Handle to "Previous" button.
	HWND			hWndPrevX5;				// Handle to "Skip back five events" button.
	HWND			hWndNext;				// Handle to "Next" button.
	HWND			hWndNextX5;				// Handle to "Skip forward five events" button.
	HWND			hWndQuit;				// Handle to "Quit" button.
	HWND			hWndMsgWarn;			// Handle to messages/warnings window.
	HWND			hWndPGVslctIn;			// Handle to "Max. Speed (in/s)" button.
	HWND			hWndPGVslctCm;			// Handle to "[Peak Velocity](cm/s)" button.

	char			msgstr[BUF_SIZE];		// Message buffer for general use

	unsigned int	wTimerID;				// ID for any timer.

	char			HistoryFile[BUF_SIZE];	// Filename of database storage file.
	FILE			*fpHistoryFile;

	LNKLST			*EqList;				// Linked list of earthquake data ("the database").
	BOOL			IsCurrentEq;			// TRUE when an earthquake is available as CurrentEq.

	EqEntry			*CurrentEq;				// The current, displayable earthquake.  NOTE:
											// This pointer not only provides easy access to
											// the current, displayed event, BUT ALSO MARKS
											// the database (linked list) position, and is
											// the only solid way to do so (because the list
											// itself is used by many routines for many
											// purposes.  (cf. MatchEvent())

	int				iListPosition = 0;		// Current earthquake sequence number (out of EqList->lstlen)
	BOOL			ListPositionOK = TRUE;	// Prevent bug at shutdown
	double			MMImax, PGAmax, PGVmax;	// Current maxima (in PERCENT G and cm/s)
	char			PercvdShake[MMI_PHRASES];// Instrumental Intensity standard phrasing:  perceived shaking
	char			PotDmg[MMI_PHRASES];	//                                             potential damage

	char			ValDispl[BUF_SIZE];		// String to display current metric ...
	char			ValDisplPGA[BUF_SIZE];	// ... and PGA substring ...
	char			ValDisplPGV[BUF_SIZE];	// ... and PGV substring .

	int				ShowEqType;				// Which earthquake type to show (big, recent, select)
	BOOL			KeepToggle;				// Toggle whether to keep or ignore an earthquake
											// so as to drop the duplicates.  Keep if TRUE.

	int				ScrnWd, ScrnHt;			// Screen size, in pixels.
	int				ScrnWdMM, ScrnHtMM;		// Screen size, in mm.
	int				nColors;				// Number of colors displayable.
	int				AspectX, AspectY;		// Aspect ratio of pixels.

	// SYSTEM_FONT
	LONG			LineHeight;				// Pixel height of a text line
	LONG			FatLineHeight;			// 1.4 * LineHeight
	LONG			CharWidthLC;			// Mean character width (lower case)
	LONG			CharWidthUC;			// Mean character width (upper case)

	// TrueType Fonts
	HFONT			hFontTR12;				// TrueType Times Roman      12pt
	HFONT			hFontTR12b;				// TrueType Times Roman Bold 12pt
	HFONT			hFontTR18b;				// TrueType Times Roman Bold 18pt
	HFONT			hFontTR22b;				// TrueType Times Roman Bold 22pt

	LONG			LineHeightTR12;
	LONG			FatLineHeightTR12;
	LONG			CharWidthLcTR12;

	LONG			LineHeightTR12b;
	LONG			FatLineHeightTR12b;
	LONG			CharWidthLcTR12b;

	LONG			LineHeightTR18b;
	LONG			FatLineHeightTR18b;
	LONG			CharWidthLcTR18b;

	LONG			LineHeightTR22b;
	LONG			FatLineHeightTR22b;
	LONG			CharWidthLcTR22b;

	HPEN			hPenDottedBlack1pt;
	HPEN			hPenDottedWhite1pt;
	HPEN			hPenSolid1pt;


	// Lables for Instrumental Intensity levels
	char		*MMIlabels[10] = {
					"I",
					"II",
					"III",
					"IV",
					"V",
					"VI",
					"VII",
					"VIII",
					"IX",
					"X+",
				};

////////////////////////////////////////////////////////////////////////////
// Interpolation for Instrumental Intensity bar graph
#define ScaleSplitMMI(split)	((int)( HALF + \
				( (double)(ScaleY + ScaleH) - \
				( ((double)ScaleH) * ((split - ONE) / (THRESH_MMI_CLIP - ONE)) ) ) \
			))


////////////////////////////////////////////////////////////////////////////
// Main Windows implementation process.

// ***** process *****
LRESULT CALLBACK WndProc(HWND	hWndMain,	// window
						 UINT	message,	// message to process
						 WPARAM	wParam,		// data 0 & 1
						 LPARAM	lParam		// data 2 (& 3)
						)
{
	PAINTSTRUCT		PntStruct;				// Paint structure
	HDC				hdc;					// Device context

	TEXTMETRIC		tm;

	RECT			rect;					// Client area stuff

	int				b_id, b_note;			// Button ID number and notification code
	HWND			b_hwnd;					// and window handle

	TCHAR			chCharCode;				// For UDPtalk()
	LPARAM			lKeyData;

	HANDLE			hInstance;

	LONG			RqstdHeight, RqstdWidth;

	// Input buffer for database history file
	char			line[BUF_SIZE];
	double			HistTimeEpoch;
	char			HistTime[LONG_STR];
	double			HistPGAch1;				// Vertical-channel   PGA.
	double			HistPGAch2;				// Horizontal-channel PGA.
	double			HistPGAch3;				// Horizontal-channel PGA.
	double			HistPGVch1;				// Vertical-channel   PGV.
	double			HistPGVch2;				// Horizontal-channel PGV.
	double			HistPGVch3;				// Horizontal-channel PGV.
	double			HistTemper;				// Temperature at earthquake time.
	int				hYr, hMo, hDa, hHr, hMn;
	double			hSec;
	EqEntry			*Evt;

	// Grab the current instance
	if (message == WM_CREATE)
		hInstance = ((LPCREATESTRUCT)lParam)->hInstance;

	// Copy the main window pointer to a global
	ghWndMain = hWndMain;

	// >>>>>>>>>>>>>>>>>>>> Main Message Handler <<<<<<<<<<<<<<<<<<<<
	switch(message) {

		// Init constants, set up Edit Control for display, set up
		// data structures and sockets ...
		case WM_CREATE:

			// Miscellaneous initializations
			nCom1FIFO = 0;
			Com1Buff[0] = '\0';
			Com1FIFO[0] = '\0';
			Site.Warn[0] = '\0';
			msgstr[0] = '\0';
			ValDispl[0] = '\0';
			ValDisplPGA[0] = '\0';
			ValDisplPGV[0] = '\0';

			// Get the size of the display--the WHOLE display--etc.
			hdc = GetDC(NULL); /////////////////////////////////////////// DC!
			ScrnWd = GetDeviceCaps(hdc, HORZRES);						//
			ScrnWdMM = GetDeviceCaps(hdc, HORZSIZE);					//
			ScrnHt = GetDeviceCaps(hdc, VERTRES);						//
			ScrnHtMM = GetDeviceCaps(hdc, VERTSIZE);					//
			nColors = GetDeviceCaps(hdc, NUMCOLORS);					//
			AspectX = GetDeviceCaps(hdc, ASPECTX);						//
			AspectY = GetDeviceCaps(hdc, ASPECTY);						//
			ReleaseDC(ghWndMain, hdc); ///////////////////////////////////

			// Create linked list for earthquake data
			EqList = ll_open();
			IsCurrentEq = FALSE;
			ShowEqType = SHOW_RECENT;
			KeepToggle = TRUE;	// Keep first earthquake seen, drop second, ...

			// Get configurables
			ConfigReader(ghWndMain);

			// Tag windows NULL prior to their creation
			hWndLogo       = NULL;
			hWndMetricEtc  = NULL;
			hWndMaxima     = NULL;
			hWndDateTime   = NULL;
			hWndWhichOf    = NULL;
			hWndSiteInfo   = NULL;
			hWndOldest     = NULL;
			hWndMostRecent = NULL;
			hWndBiggest    = NULL;
			hWndPrev       = NULL;
			hWndPrevX5     = NULL;
			hWndNext       = NULL;
			hWndNextX5     = NULL;
			hWndPGVslctIn  = NULL;
			hWndPGVslctCm  = NULL;

			// Get font info; set for text units /////////////////////////
			hdc = GetDC(ghWndMain);										// DC!
			SetMapMode(hdc, MM_TEXT);									//
			GetTextMetrics(hdc, &tm);									//
																		//
			LineHeight = tm.tmExternalLeading + tm.tmHeight;			//
			FatLineHeight = (LONG)(										//
				(double)1.4 * (double)LineHeight + HALF);				//
			CharWidthLC = tm.tmAveCharWidth;							//
			CharWidthUC = (LONG)(										//
				HALF + (double)(tm.tmAveCharWidth) * (double)1.5);		//
																		//
			// Create TrueType Fonts /////////////////////////////////////
			hFontTR12 = EzCreateFont (hdc, "Times New Roman",			//
				120, 0, 0, 0);											//
			hFontTR12b = EzCreateFont (hdc, "Times New Roman Bold",		//
				120, 0, 0, 0);											//
			hFontTR18b = EzCreateFont (hdc, "Times New Roman Bold",		//
				180, 0, 0, 0);											//
			hFontTR22b = EzCreateFont (hdc, "Times New Roman Bold",		//
				220, 0, 0, 0);											//
																		//
			SelectObject(hdc, hFontTR12);								//
			GetTextMetrics(hdc, &tm);									//
			LineHeightTR12 = tm.tmExternalLeading + tm.tmHeight;		//
			FatLineHeightTR12 = (LONG)(									//
				(double)1.3 * (double)LineHeightTR12 + HALF);			//
			CharWidthLcTR12 = tm.tmAveCharWidth;						//
																		//
			SelectObject(hdc, hFontTR12b);								//
			GetTextMetrics(hdc, &tm);									//
			LineHeightTR12b = tm.tmExternalLeading + tm.tmHeight;		//
			FatLineHeightTR12b = (LONG)(								//
				(double)1.3 * (double)LineHeightTR12b + HALF);			//
			CharWidthLcTR12b = tm.tmAveCharWidth;						//
																		//
			SelectObject(hdc, hFontTR18b);								//
			GetTextMetrics(hdc, &tm);									//
			LineHeightTR18b = tm.tmExternalLeading + tm.tmHeight;		//
			FatLineHeightTR18b = (LONG)(								//
				(double)1.2 * (double)LineHeightTR18b + HALF);			//
			CharWidthLcTR18b = tm.tmAveCharWidth;						//
																		//
			ScaleWidth = (int)(HALF + ((double)SCALE_WIDTH *			//
				(double)CharWidthLcTR18b / (double)9.0));				//
			LableWidth = (int)(HALF + ((double)LABLE_WIDTH *			//
				(double)CharWidthLcTR18b / (double)9.0));				//
																		//
			SelectObject(hdc, hFontTR22b);								//
			GetTextMetrics(hdc, &tm);									//
			LineHeightTR22b = tm.tmExternalLeading + tm.tmHeight;		//
			FatLineHeightTR22b = (LONG)(								//
				(double)1.2 * (double)LineHeightTR22b + HALF);			//
			CharWidthLcTR22b = tm.tmAveCharWidth;						//
																		//
			// Create special pens ///////////////////////////////////////
			hPenDottedBlack1pt = CreatePen(PS_DOT, 1, BLACK);			//
			hPenDottedWhite1pt = CreatePen(PS_DOT, 1, WHITE);			//
			hPenSolid1pt = CreatePen(PS_SOLID, 1, BLACK);				//
			ReleaseDC(ghWndMain, hdc); ///////////////////////////////////

			// Lay out Main window ///////////////////////////////////////////////////////////
			RqstdHeight = GUTTER_SIZE + RqstdBarH + GUTTER_SIZE + IntTtlH + GUTTER_SIZE;	//
			// Leave room for bargraph at least twice the width of the scale.				//
			RqstdWidth = GUTTER_SIZE + CONTROL_WIDTH + GUTTER_SIZE							//
				+ RqstdBarW + GUTTER_SIZE + ScaleWidth + (2 * LableWidth) + GUTTER_SIZE;	//
			MainHeight = MAX(DEF_WIDTH, RqstdHeight) + CAPTIONH + 2 * WNDBDR;				//
			MainWidth = MAX(DEF_HEIGHT, RqstdWidth) + 2 * WNDBDR;							//
			ObjectLayout();		// Approximate, to give MakeControlls() values				//
			//////////////////////////////////////////////////////////////////////////////////

			// Create all the objects
			MakeControls(hInstance);

			// Put window where requested (and the real call to ObjectLayout()).
			MoveWindow(ghWndMain,	// are set by MakeControls() ).
				OriginX, OriginY,
				MainWidth, MainHeight,
				TRUE);

			// For good measure, ask back what the result was
			GetClientRect(ghWndMain, &rect);
			CAWidth = rect.right - rect.left;
			CAHeight = rect.bottom - rect.top;
			GetWindowLayout(ghWndMain, &WMSize);
			// Hereafter, use WMSize.CaptionH and WMSize.WndBdr
			// in place of CAPTIONH and WNDBDR.

			// Start cleanup-check interval timer
			if (TIMER_CLEAN != SetTimer(ghWndMain, TIMER_CLEAN, CLEANUP_CHECK,
					(TIMERPROC)NULL)) {
				ErrExit("Could not initiate\ncleanup-check timer.");
			}

			// Link to COM1 for input.
			Com1 = StartCom1();
			if (Com1 == INVALID_HANDLE_VALUE)	// redundant
				ErrExit("Could not initiate\nCOM1 for reading.");

			// Start input-check interval timer
			if (TIMER_COM1 != SetTimer(ghWndMain, TIMER_COM1, COM1_CHECK,
					(TIMERPROC)NULL)) {
				ErrExit("Could not initiate\ninput-check timer.");
			}

			// Open any HistoryFile and read into earthquake database, EqList.
			fpHistoryFile = fopen(HistoryFile, "r");
			if (NULL != fpHistoryFile) {

				while(NULL != fgets(line, BUF_SIZE, fpHistoryFile)) {

					// Read in an earthquake (Sa will be %7.2lf)
					strncpy(HistTime, line, 20);
					HistTime[20]='\0';
					sscanf(line, "%02d/%02d/%04d  %02d:%02d:%02lf  %7lf %7lf %7lf  %6lf %6lf %6lf  %6lf",
						&hMo, &hDa, &hYr, &hHr, &hMn, &hSec,
						&HistPGAch1, &HistPGAch2, &HistPGAch3,		// %g
						&HistPGVch1, &HistPGVch2, &HistPGVch3,		// cm/s
						&HistTemper);
					HistTimeEpoch = timvar(hYr, hMo, hDa, hHr, hMn, hSec, GREGORIAN);

					// Create item, move to end of list, and add earthquake to list
					Evt = CreateEqItem(HistTimeEpoch, HistTime,
						HistPGAch1, HistPGAch2, HistPGAch3,		// %g
						HistPGVch1, HistPGVch2, HistPGVch3,		// cm/s
						HistTemper);
					ll_tail(EqList);		// This moves the event pointer, so ...
					ll_add(EqList, (char *)Evt);

				}

				// Force SideBar to the latest event if there are any
				if (EqList->lstlen > 0) {
					ShowEqType = SHOW_RECENT;
					HighlightEventButton();
					ll_tail(EqList);
					GetCurrentEq();
				}

				// Last, remove any aged-out earthquakes
				fclose(fpHistoryFile);
				CleanMemory(Site.MemoryDuration);
			}

			// Default to in/s for PGV
			PGVscale = INCHES_PER_S;
			HighlightMeasureButton();

			// Null text for warnings box:
			ErrWarn("Messages:\r\n   Intensity is not Mangitude.");

			break;
		// WM_CREATE

		// Change window size
		case WM_SIZE:
			MainWidth = LOWORD(lParam);
			MainHeight = HIWORD(lParam);
			ObjectLayout();

			if (hWndLogo != NULL)
				MoveWindow(hWndLogo,
					LogoX, LogoY,
					LogoW, LogoH,
					TRUE);


			if (hWndMetricEtc != NULL)
				MoveWindow(hWndMetricEtc,
					MetricEtcX, MetricEtcY,
					MetricEtcW, MetricEtcH,
					TRUE);

			if (hWndMaxima != NULL)
				MoveWindow(hWndMaxima,
					MaximaX, MaximaY,
					MaximaW, MaximaH,
					TRUE);


			if (hWndDateTime != NULL)
				MoveWindow(hWndDateTime,
					DateTimeX, DateTimeY,
					DateTimeW, DateTimeH,
					TRUE);

			if (hWndWhichOf != NULL)
				MoveWindow(hWndWhichOf,
					WhichOfX, WhichOfY,
					WhichOfW, WhichOfH,
					TRUE);

			if (hWndOldest != NULL)
				MoveWindow(hWndOldest,
					OldestX, OldestY,
					OldestW, OldestH,
					TRUE);

			if (hWndMostRecent != NULL)
				MoveWindow(hWndMostRecent,
					MostRecentX, MostRecentY,
					MostRecentW, MostRecentH,
					TRUE);

			if (hWndBiggest != NULL)
				MoveWindow(hWndBiggest,
					BiggestX, BiggestY,
					BiggestW, BiggestH,
					TRUE);

			if (hWndPrev != NULL)
				MoveWindow(hWndPrev,
					PrevX, PrevY,
					PrevW, PrevH,
					TRUE);

			if (hWndPrevX5 != NULL)
				MoveWindow(hWndPrevX5,
					PrevX5X, PrevX5Y,
					PrevX5W, PrevX5H,
					TRUE);

			if (hWndNext != NULL)
				MoveWindow(hWndNext,
					NextX, NextY,
					NextW, NextH,
					TRUE);

			if (hWndNextX5 != NULL)
				MoveWindow(hWndNextX5,
					NextX5X, NextX5Y,
					NextX5W, NextX5H,
					TRUE);

			if (hWndSiteInfo != NULL)
				MoveWindow(hWndSiteInfo,
					SiteInfoX, SiteInfoY,
					SiteInfoW, SiteInfoH,
					TRUE);

			if (hWndQuit != NULL)
				MoveWindow(hWndQuit,
					QuitX, QuitY,
					QuitW, QuitH,
					TRUE);

			if (hWndMsgWarn != NULL)
				MoveWindow(hWndMsgWarn,
					MsgWarnX, MsgWarnY,
					MsgWarnW, MsgWarnH,
					TRUE);

			if (hWndPGVslctIn != NULL)
				MoveWindow(hWndPGVslctIn,
					PGVslctInX, PGVslctInY,
					PGVslctInW, PGVslctInH,
					TRUE);

			if (hWndPGVslctCm != NULL)
				MoveWindow(hWndPGVslctCm,
					PGVslctCmX, PGVslctCmY,
					PGVslctCmW, PGVslctCmH,
					TRUE);
			
			break;

		// Trap keyboard hits.  Result currently ignored.
		case WM_CHAR:
			chCharCode = (char)wParam;		// character code 
			lKeyData = lParam;				// key data 
			break;

		// Paint client area (required to fill the window)
		// This is where everything is really drawn and written.
		case WM_PAINT:

			hdc = BeginPaint(ghWndMain, &PntStruct); ///////////////////////// DC!
																			//
			// Write any current earthquake information						//
			if (IsCurrentEq) {													//
				SetWindowText(hWndDateTime, CurrentEq->chTime);				//
																			//
				sprintf(ValDispl, "Earthquake %d of %d",					//
					iListPosition, EqList->lstlen);							//
				SetWindowText(hWndWhichOf, ValDispl);						//
																			//
				///	///	///	///	///	///	///	///	///	///	///	///	///	///	///	//
				// Write Instrumental Intensity in primary metric window.	//
				// MMImax = PGAVtoMMI(((PGAmax / HH) * ONE_G ), PGVmax);	// Redundant?
				MMIphrases(MMImax, PercvdShake, PotDmg);					//
				sprintf(ValDispl,											//
					"At this site:\r\n  Perceived Shaking is\r\n    \"%s.\"\r\n  Potential Damage is\r\n    \"%s.\"\r\n  Intensity is \"%s\".",		//
					PercvdShake, PotDmg,									//
					MMIlabels[MIN(9, MAX(0, ((int)(MMImax + HALF) - 1)))]);	//
				SetWindowText(hWndMetricEtc, ValDispl);						//
																			//
				// Write PGA and PGV near bottom of control area.			//
				if (PGAmax >= ONE)											//
					sprintf(ValDisplPGA,									//
						"Maximum force\r\n  is %.0lf %% of gravity.\r\n",	//
						PGAmax);											//
				else if (PGAmax >= (double)0.1)								//
					sprintf(ValDisplPGA,									//
						"Maximum force\r\n  is %.1lf %% of gravity.\r\n",	//
						PGAmax);											//
				else														//
					sprintf(ValDisplPGA,									//
						"Maximum force\r\n  is %.2lf %% of gravity.\r\n",	//
						PGAmax);											//
																			//
				if (PGVscale == INCHES_PER_S) {								//
					if ((PGVmax / INtoCM) >= ONE)							//
						sprintf(ValDisplPGV,								//
							"Maximum speed\r\n  is %.0lf inches/sec.",		//
							PGVmax / INtoCM);								//
					else if ((PGVmax / INtoCM) >= (double)0.1)				//
						sprintf(ValDisplPGV,								//
							"Maximum speed\r\n  is %.1lf inches/sec.",		//
							PGVmax / INtoCM);								//
					else													//
						sprintf(ValDisplPGV,								//
							"Maximum speed\r\n  is %.2lf inches/sec.",		//
							PGVmax / INtoCM);								//
				}															//
																			//
				else {														//
					if (PGVmax >= ONE)										//
						sprintf(ValDisplPGV,								//
							"Maximum speed\r\n  is %.0lf cm/sec.",			//
							PGVmax);										//
					else if (PGVmax >= (double)0.1)							//
						sprintf(ValDisplPGV,								//
							"Maximum speed\r\n  is %.1lf cm/sec.",			//
							PGVmax);										//
					else													//
						sprintf(ValDisplPGV,								//
							"Maximum speed\r\n  is %.2lf cm/sec.",			//
							PGVmax);										//
				}															//
																			//
				(void)strcpy(ValDispl, ValDisplPGA);						//
				(void)strcat(ValDispl, ValDisplPGV);						//
																			//
				SetWindowText(hWndMaxima, ValDispl);						//
				///	///	///	///	///	///	///	///	///	///	///	///	///	///	///	//
																			//
			}																//
			else {															//
				SetWindowText(hWndDateTime, "Database Empty");				//
				SetWindowText(hWndWhichOf, "No Earthquakes");				//
				SetWindowText(hWndMetricEtc, " ");							//
				MMImax = ZIP;												//
			}																//
																			//
																			//
			// In any case, write site information							//
			sprintf(msgstr, "%s\r\nSite \"%s\"",							//
				Site.StationSite, Site.TREMOR_SN);							//
			SetWindowText(hWndSiteInfo, msgstr);							//
																			//
																			//
			///	///	///	///	///	///	///	///	///	///	///	///	///	///	///	///	//
			// Draw current histogram bar									//
			(void)DrawMMIBar(hdc);											//
			///	///	///	///	///	///	///	///	///	///	///	///	///	///	///	///	//
																			//
			// Draw USGS Logo												//
			DrawBitmap(hdc, hLogo, LogoX, LogoY);							//
																			//
			// Make scale bar and space for the histogram bar				//
			DrawScalesLables(hdc, MMImax);									//
																			//
																			//
			EndPaint(ghWndMain, &PntStruct); /////////////////////////////////

			break;

		// Custom Messages ///////////////////////////////////////////////////
																			//
		// Something about display has changed, so refresh whole thing		//
		case WM_UPDATE_WIN:	  												//
			InvalidateRect(ghWndMain, NULL, TRUE);	// Whole client area	//
			UpdateWindow(ghWndMain);				// WM_PAINT right now	//
			break;				 		 									//
																			//
		// Catch button hits (pass it on if unknown)						//
		case WM_COMMAND:  													//
			b_hwnd = (HWND)lParam;											//
			b_id = LOWORD(wParam);											//
			b_note = HIWORD(wParam);										//
			if (															//
					b_id == IDB_OLDEST        ||							//
					b_id == IDB_MOST_RECENT   ||							//
					b_id == IDB_BIGGEST       ||							//
					b_id == IDB_PREVIOUS      ||							//
					b_id == IDB_PREVIOUS_X5   ||							//
					b_id == IDB_NEXT          ||							//
					b_id == IDB_NEXT_X5       ||							//
					b_id == IDB_QUIT          ||							//
					b_id == IDB_PGV_SELECT_IN ||							//
					b_id == IDB_PGV_SELECT_CM								//
				) {															//
				Controls(b_hwnd, b_id, b_note);								//
				break;														//
			}																//
			else {															//
				return (DefWindowProc(ghWndMain, message, wParam, lParam));	//
			}																//
																			//
		// End Custom Messages ///////////////////////////////////////////////

		// Respond to periodic timer by cleaning up earthquake history.
		case WM_TIMER:
			wTimerID = wParam;		// timer identifier

			if (wTimerID == TIMER_CLEAN) {
				if (!KillTimer(ghWndMain, TIMER_CLEAN)) {
					ErrWarn("Could not kill cleanup-check timer.\r\n");
				}
				wTimerID = SetTimer(ghWndMain, TIMER_CLEAN, CLEANUP_CHECK,
					(TIMERPROC)NULL); 

				// Update memory
				CleanMemory(Site.MemoryDuration);
			}

			else if (wTimerID == TIMER_COM1) {
				if (!KillTimer(ghWndMain, TIMER_COM1)) {
					ErrExit("Could not kill\r\ninput-check timer.");
				}
				wTimerID = SetTimer(ghWndMain, TIMER_COM1, COM1_CHECK,
					(TIMERPROC)NULL);

				// Read any available input <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
				if (!ReadFile(Com1,					// immediate return
						(LPVOID)Com1Buff, (DWORD)COM1_BUFSIZE,
						&nBytesRead, NULL)) {
					sprintf(msgstr, "Read from COM1 failed (%d).", GetLastError());
					ErrExit(msgstr);
				}
				Com1Buff[nBytesRead] = '\0';	// Force string termination

				// Add the data to the FIFO ...
				if (nBytesRead) {
					if (!FIFOstuff(&nCom1FIFO, COM1_BUFSIZE, Com1FIFO,
							Com1Buff, nBytesRead)) {
						ErrWarn("Discarded some data due to\r\n  FIFO overflow.\r\n");
					}
				}

				// ... and if there is a complete line in the FIFO ...
				while ( NULL != (FIFOline = FIFOgets(&nCom1FIFO, Com1FIFO)) ) {
					// ... process it for trigger content.
					GrabTrg(FIFOline);
				}
			}

			else {
				ErrWarn("Invalid timer.\r\n");
			}

			break;

		// Bug out
		case WM_DESTROY:

			// And write out final earthquake database
			(void)WriteDataBase();

			ListPositionOK = FALSE;

			// Close COM1
			CloseHandle(Com1);

			// Remove linked list of earthquake data
			ll_close(EqList, DeleteEqItem);

			// Remove Fonts, pens, bitmaps, ...
			DeleteObject(hFontTR12);
			DeleteObject(hFontTR12b);
			DeleteObject(hFontTR18b);
			DeleteObject(hFontTR22b);
			DeleteObject(hPenDottedBlack1pt);
			DeleteObject(hPenDottedWhite1pt);
			DeleteObject(hPenSolid1pt);
			DeleteObject(hLogo);

			// Windows clean up
			PostQuitMessage(0);

			break;

		// Pass message on if not processed here
		default:
			return (DefWindowProc(ghWndMain, message, wParam, lParam));
	}

	return ((LRESULT)0);
}	// WndProc()


////////////////////////////////////////////////////////////////////////////
// Check for a match of MMItmp withing the list TopMMIs[].  If any matches,
// return TRUE, otherwise FALSE.

// ***** process *****
BOOL
CheckMMImatches(double MMItmp, double TopMMIs[N2PROTECT])
{
	int			ii;
	BOOL		rtn;

	rtn = FALSE;
	for (ii = 0 ; ii < N2PROTECT ; ii++) {
		if (MMItmp == TopMMIs[ii]) {
			rtn = TRUE;
			break;
		}
	}

	return(rtn);
}	// CheckMMImatches()


////////////////////////////////////////////////////////////////////////////
// Discard all earthquakes older than MemoryDuration seconds, except for
// the N2PROTECT largest events of any size (and any tieing those MMIs)
// and also all events of intensity at least THRESH_MMI_SAVE.

// ***** process *****
void
CleanMemory(double MemoryDuration)
{
	EqEntry			*TempEq;				// Earthquake to consider.
	double			TimeNow;				// Current time,
	SYSTEMTIME		LocalTime;				// Yup.

	// NULL database
	if (EqList->lstlen == 0) {
		GetCurrentEq();		// Sets flags and values right
		return;
	}

	// Tag the largest events to protect them from deletion.
	FindLargestEvents();

	// Note current local time
	GetLocalTime(&LocalTime);
	TimeNow = timvar(
		(int)LocalTime.wYear,
		(int)LocalTime.wMonth,
		(int)LocalTime.wDay,
		(int)LocalTime.wHour,
		(int)LocalTime.wMinute,
		(double)LocalTime.wSecond + (double)LocalTime.wMilliseconds / (double)1000.,
		GREGORIAN);


	// Run through entire database, eliminating old, small earthquakes.
	ll_head(EqList);					// Null earthquake (list head is always empty)
	while (ll_next(EqList)) {
		TempEq = (EqEntry *)ll_ret(EqList);

		if (	((TimeNow - TempEq->Time) > MemoryDuration) &&	// It's old
				(!(TempEq->LargestMMIs)) &&						// It's not a "largest N2PROTECT"
				(!(TempEq->MMI >= 6.0))							// It's not MMI VI or over
			) {
			// Delete old earthquake
			DeleteEqItem(ll_delete(EqList));
		}
	}

	// Reposition database back to current earthquake, and
	// update earthquake display.  Takes care of empty event
	// lists right too.  If fails to find CurrentEq, defaults
	// to "most recent" and trys again.
	GetCurrentEq();

}	// CleanMemory()


////////////////////////////////////////////////////////////////////////////
// Temperature and gain compensation for TREMOR data.  Given ADC counts, it
// returns cm/s^2.  "ist" is index to Sites[]; "ich" is channel (V,N,E=1,2,3);
// "temperature" is degrees C; "ADCcounts" is raw acceleration value from ADC.
// Returns +/-Inf for NULL_WAVE_P and NULL_WAVE_M.

// ***** process *****
double
Compensate(int ich,
		   double temperature,
		   short int ADCcounts)
{
	double	voltage, gals;
	double	Sensitivity, TCS, Tref;

	if (ADCcounts >= NULL_WAVE_P) {
		return(HUGE);
	}
	if (ADCcounts <= NULL_WAVE_M) {
		return(-HUGE);
	}

	// Get the necessary compensation coefficients
	switch (ich) {
		case (1): {
			Sensitivity = Site.V_Sensitivity;
			TCS = Site.V_TCS;
			Tref = Site.V_Tref;
			break;
		}
		case (2): {
			Sensitivity = Site.N_Sensitivity;
			TCS = Site.N_TCS;
			Tref = Site.N_Tref;
			break;
		}
		case (3): {
			Sensitivity = Site.E_Sensitivity;
			TCS = Site.E_TCS;
			Tref = Site.E_Tref;
			break;
		}
		default: {
			ErrWarn("Invalid channel number in Compensate().\r\n  Default values used.\r\n");
			Sensitivity = (double)0.00254929;	// 2.5 V/g
			TCS = -(double)0.0015;				// -15%/100C is average for a large set
			Tref = (double)22.0;				// Good guess
		}
	}

	// ADC counts to voltage (and degain it) ...
	voltage = (double)ADCcounts * ADCtoV / (double)16.0;

	// ... to uncompensated acceleration ...
	gals = voltage / Sensitivity;
	
	// ... to compensated acceleration ...
	gals *= (ONE + TCS * (temperature - Tref));

	return ((double)gals);
}	// Compensate()


///////////////////////////////////////////////////////////////////////////////
// Convert and compensate PGV results.  Works because PGV is, for a given sample
// rate, just a linear combination of (unconverted, uncompensated) acceleration,
// while conversion and compensation is via a series of scalar multipliers.  So
// the raw PGV result is the true result times this series of scalars, which
// here are removed.

// ***** process *****
double
CompPGV(long RawPGV, double Sensitivity,
		double TCS, double Tref,
		double temperature, double sps)
{
	double		voltage, TruePGV;

	// Convert to voltage (and degain it) ...
	voltage = RawPGV * ADCtoV / SIXTEEN;

	// ... to uncompensated velocity (cm/s) ...
	TruePGV = voltage / Sensitivity / sps;
	
	// ... to compensated velocity ...
	TruePGV *= (ONE + TCS * (temperature - Tref));

	return (TruePGV);
}	// CompPGV()


///////////////////////////////////////////////////////////////////////////////
// Convert and compensate Sa results.  Works because Sa for a given sample
// rate, damping, and period is simply a linear function of (unconverted,
// uncompensated) acceleration, while conversion and compensation is via a
// series of scalar multipliers.  So the raw Sa result is the true result
// times this series of scalars, which here are removed.

// ***** process *****
double
CompSa(long RawSa, double Sensitivity,
		double TCS, double Tref,
		double temperature, double scale)
{
	double		voltage, TrueSa, RawSaD;

	// Unshift it
	RawSaD = (double)RawSa / pow(TWO, scale);

	// Convert to voltage (and degain it) ...
	voltage = RawSaD * ADCtoV / SIXTEEN;

	// ... to uncompensated acceleration (gal) ...
	TrueSa = voltage / Sensitivity;
	
	// ... to compensated acceleration ...
	TrueSa *= (ONE + TCS * (temperature - Tref));

	return (TrueSa);
}	// CompSa()


////////////////////////////////////////////////////////////////////////////
// Read in "C:\SideBarConfig.txt" and set configurable variables.

// ***** process *****
void
ConfigReader()
{
	char	line[BUF_SIZE];
	char	*lp;
	int		vptr = 0;		// points to current input variable
	FILE	*fp;
	int		ii;
	double	aa;

	fp = fopen("C:\\SideBarConfig.txt", "r");
	if (NULL == fp) {
		ErrExit("Could not open \"C:\\SideBarConfig.txt\".");
	}

	while (NULL != fgets(line, BUF_SIZE, fp)) {	// Quit at EOF.
		lp = strpbrk(line, " \t#\r\n");		// Find first comments/whitespace or ...
		if (lp == NULL) {
			lp = line + strlen(line);	// ... end of line, then ...
		}
		*lp = '\0';						// ... strip all junk off, and ...
		if (strlen(line) > 0) {			// ... read a value from nonblank line.
			vptr++;

			switch(vptr) {
			case(1):	// HistoryFile
				strcpy(HistoryFile, line);
				break;
			case(2):	// OriginX
				OriginX = (LONG)atol(line);
				break;
			case(3):	// OriginY
				OriginY = (LONG)atol(line);
				break;
			case(4):	// RqstdBarH
				RqstdBarH = atol(line);
				break;
			case(5):	// RqstdBarW
				RqstdBarW = atol(line);
				break;
			case(6):	// MemoryDuration
				aa = atof(line);
				// Limit memory length to range (15 minutes ... 7 days)
				Site.MemoryDuration = RANGE(0.25, aa, 168.0);
				Site.MemoryDuration *= (double)3600.;	// hours --> seconds
				break;
			case(7):	// MaxDBaseLen
				Site.MaxDBaseLen = abs(atoi(line));
				break;
			case(8):	// StationSite
				strcpy(Site.StationSite, line);
				for (ii = 0 ; ii < (int)strlen(Site.StationSite) ; ii++) {
					if (Site.StationSite[ii] == '_')
						Site.StationSite[ii] = ' ';
				}
				break;
			case(9):	// TREMOR_SN
				strcpy(Site.TREMOR_SN, line);
				break;
			case(10):	// latitude
				Site.latitude = atof(line);
				break;
			case(11):	// longitude
				Site.longitude = atof(line);
				break;
			case(12):	// elevation
				Site.elevation = atof(line);
				break;
			case(13):	// N_azimuth
				Site.N_azimuth = atof(line);
				break;
			case(14):	// N_azm_err
				Site.N_azm_err = atof(line);
				break;

			case(15):	// TimeFudge
				Site.TimeFudge = atof(line);
				break;

			case(16):	// V_ICS_SN
				strcpy(Site.V_ICS_SN, line);
				break;
			case(17):	// V_Sensitivity
				Site.V_Sensitivity = -atof(line) / ONE_G;	// V/g --> V/gal
				break;
			case(18):	// V_TCS
				Site.V_TCS = (atof(line) / HH) / HH;		// %/100C --> fraction/C
				break;
			case(19):	// V_Tref
				Site.V_Tref = atof(line);
				break;

			case(20):	// N_ICS_SN
				strcpy(Site.N_ICS_SN, line);
				break;
			case(21):	// N_Sensitivity
				Site.N_Sensitivity = -atof(line) / ONE_G;	// V/g --> V/gal
				break;
			case(22):	// N_TCS
				Site.N_TCS = (atof(line) / HH) / HH;		// %/100C --> fraction/C
				break;
			case(23):	// N_Tref
				Site.N_Tref = atof(line);
				break;

			case(24):	// E_ICS_SN
				strcpy(Site.E_ICS_SN, line);
				break;
			case(25):	// E_Sensitivity
				Site.E_Sensitivity = -atof(line) / ONE_G;	// V/g --> V/gal
				break;
			case(26):	// E_TCS
				Site.E_TCS = (atof(line) / HH) / HH;		// %/100C --> fraction/C
				break;
			case(27):	// E_Tref
				Site.E_Tref = atof(line);
				break;

			default:
				ErrExit("Too many variables in \"C:\\SideBarConfig.txt\".");
			}	// switch()
		}		// if()
	}			// while()

	fclose(fp);

	if (vptr != 27) {
		ErrExit("Wrong number of variables in\n\"C:\\SideBarConfig.txt\".");
	}
}	// ConfigReader()


////////////////////////////////////////////////////////////////////////////
// Respond to control (i.e., button) inputs.

// ***** process *****
void
Controls(HWND b_hwnd, int b_id, int b_note)
{
	int		ii, nskip = 1;						// Skip one event unless otherwise instructed

	switch (b_id) {
		case IDB_OLDEST: {

			ShowEqType = SHOW_ANCIENT;			// Show oldest earthquake
			HighlightEventButton();
			GetCurrentEq();
			
			break;
		}

		case IDB_MOST_RECENT: {

			ShowEqType = SHOW_RECENT;			// Show current earthquake
			HighlightEventButton();
			GetCurrentEq();
			
			break;
		}

		case IDB_BIGGEST: {

			ShowEqType = SHOW_BIGGEST;			// Show biggest earthquake
			HighlightEventButton();
			GetCurrentEq();
			
			break;
		}


		case IDB_PREVIOUS_X5:
			nskip = 5;							// Skip 5, not the usual 1 events
		case IDB_PREVIOUS: {

			ShowEqType = SHOW_SELECTED_P;		// Show previous earthquake
			HighlightEventButton();

			if (EqList->lstlen > 0) {
				GetCurrentEq();					// Position the list to the current event; ...

				for (ii = 0 ; ii < nskip ; ii++) {
					if (!ll_prev(EqList)) {		// ... step back; if bump head, it's null so ...
						ll_head(EqList);		// ... ( after making sure we're really there) ...
						(void)ll_next(EqList);	// ... go past head to first real earthquake ...
						ShowEqType = SHOW_ANCIENT;// ... and switch to "oldest" display.
						HighlightEventButton();
					}
				}

				// Mark current earthquake and go to it
				CurrentEq = (EqEntry *)ll_ret(EqList);
				GetCurrentEq();
			}
			
			break;
		}

		case IDB_NEXT_X5:
			nskip = 5;							// Skip 5, not the usual 1 events
		case IDB_NEXT: {

			ShowEqType = SHOW_SELECTED_N;		// Show next earthquake
			HighlightEventButton();

			if (EqList->lstlen > 0) {
				GetCurrentEq();					// Position the list to the current event; ...

				for (ii = 0 ; ii < nskip ; ii++) {
					if (!ll_next(EqList)) {		// ... step forward ...
						ShowEqType = SHOW_RECENT;// ... if run off end, switch to "latest"
						HighlightEventButton();
					}
				}

				// Mark current earthquake and go to it
				CurrentEq = (EqEntry *)ll_ret(EqList);
				GetCurrentEq();
			}
			
			break;
		}

		// Exit
		case IDB_QUIT: {
			// Ask for user verification
			if (IDYES == MessageBox(ghWndMain,
					"This will kill SideBar and its\ndatabase.  Are you certain?",
					"Quit TREMOR_Receiver()?",
					MB_ICONEXCLAMATION | MB_YESNO | MB_DEFBUTTON2)) {
				// Nuke it all
				SendMessage(ghWndMain, WM_DESTROY, (WPARAM)0, (LPARAM)0);
			}
			break;
		}

		// Choose PGV-display mode (in/s)
		case IDB_PGV_SELECT_IN: {

			PGVscale = INCHES_PER_S;
			HighlightMeasureButton();
			if (ShowEqType == SHOW_BIGGEST)
				GetCurrentEq();
			
			break;
		}

		// Choose PGV-display mode (cm/s)
		case IDB_PGV_SELECT_CM: {

			PGVscale = CM_PER_S;
			HighlightMeasureButton();
			if (ShowEqType == SHOW_BIGGEST)
				GetCurrentEq();
			
			break;
		}


		// Should never reach this point
		default: {
			ErrExit("Programming error\nin/using Controls().");
		}
	} // switch()

	// Signal display update since a change in earthquake or
	// parameter was requested.
	SendMessage(ghWndMain, WM_UPDATE_WIN, (WPARAM)0, (LPARAM)0);

}	// Controls()


////////////////////////////////////////////////////////////////////////////
// Create an earthquake-record item to add to the linked list.
// Crash ungracefully if failed.
//
// DOES NOT set values of CurrentEq or IsCurrentEq.  User MUST deal with
// these after calling CreateEqItem() in a case-appropriate way.

// ***** process *****
EqEntry *
CreateEqItem(
		double		Time,
		char		*chTime,
		double		PGAch1, double PGAch2, double PGAch3,		// %g
		double		PGVch1, double PGVch2, double PGVch3,		// cm/s
		double		Temperature)
{
static	EqEntry		*NewItemPtr;
		double		MMItmp, PGAtmp, PGVtmp;	// Local maxima

	// Do not allow list to grow longer than MaxDBaseLen
	while (EqList->lstlen >= Site.MaxDBaseLen) {
		if (!DeleteSmallestEq())
			ErrExit("Programming error #1 while allocating\nevent space in CreateEqItem().");
	}


	// Allocate new item
	while (NULL ==
			( NewItemPtr = (EqEntry *)malloc((size_t)sizeof(EqEntry)) )
			) {
		if (!DeleteSmallestEq())
			ErrExit("Programming error #2 while allocating\nevent space in CreateEqItem().");
	}

	PGAtmp = MAX3(PGAch1, PGAch2, PGAch3);
	PGVtmp = MAX3(PGVch1, PGVch2, PGVch3);
	MMItmp = PGAVtoMMI(((PGAtmp / HH) * ONE_G ), PGVtmp);

	// Fill new item
	NewItemPtr->Time = Time;
	(void)strncpy(NewItemPtr->chTime, chTime, LONG_STR);
	NewItemPtr->PGAch1      = PGAch1;
	NewItemPtr->PGAch2      = PGAch2;
	NewItemPtr->PGAch3      = PGAch3;
	NewItemPtr->PGVch1      = PGVch1;
	NewItemPtr->PGVch2      = PGVch2;
	NewItemPtr->PGVch3      = PGVch3;
	NewItemPtr->MMI         = MMItmp;
	NewItemPtr->LargestMMIs = FALSE;		// Place holder only (cf. FindLargestEvents())
	NewItemPtr->Temperature = Temperature;

	return(NewItemPtr);
}	// CreateEqItem()


////////////////////////////////////////////////////////////////////////////
// Delete an earthquake-record item from the linked list.

// ***** process *****
void
DeleteEqItem(char *OldItemPtr)
{
	free((EqEntry *)OldItemPtr);
}	// DeleteEqItem()


////////////////////////////////////////////////////////////////////////////
// Delete the earthquake with the smallest PGV from the linked list.
// Returns TRUE if successfull, FALSE if it fails to delete.

// ***** process *****
BOOL
DeleteSmallestEq()
{
	EqEntry			*TempEq;			// Earthquake to consider.
	double			PGVmin;				// Minimum

	// Find value of smallest PGV in entire earthquake list
	ll_head(EqList);					// Null earthquake
	PGVmin = HUGE;
	while (ll_next(EqList)) {
		TempEq = (EqEntry *)ll_ret(EqList);
		PGVmin = MIN(PGVmin, MAX3(TempEq->PGVch1, TempEq->PGVch2, TempEq->PGVch3));
	}

	// Find oldest matching earthquake, and delete it.
	ll_head(EqList);					// Null earthquake
	while (ll_next(EqList)) {
		TempEq = (EqEntry *)ll_ret(EqList);
		if (PGVmin == MAX3(TempEq->PGVch1, TempEq->PGVch2, TempEq->PGVch3)) {
			DeleteEqItem(ll_delete(EqList));

			// Update earthquake display
			if (TempEq == CurrentEq &&
					(ShowEqType == SHOW_SELECTED_P || ShowEqType == SHOW_SELECTED_N)
				) {
				ShowEqType = SHOW_RECENT;
				HighlightEventButton();
			}

			// Reposition linked list to current and return.
			// (If GetCurrentEq() fails, it defaults to
			// "most recent" and trys again,)
			GetCurrentEq();

			return(TRUE);
		}
	}

	// Otherwise, there must have been a problem
	// (ran out of earthquakes to delete; no earthquake matched PGVmin).
	return(FALSE);

}	// DeleteSmallestEq()


////////////////////////////////////////////////////////////////////////////
// Find MMImax and draw the histogram bar for Instrumental Intensity
// (MMImax).  Returns TRUE if plotted on scale, FALSE if clipped.
// (Return value currently ignored.)

// ***** process *****
BOOL
DrawMMIBar(HDC hdc)
{
	HBRUSH		hBrush;				// Brush
	RECT		rctngl;				// Rectangle
	int			left, top, topb, right, bottom;
	int			SMCred, SMCgreen, SMCblue;


	// May be called before data exist.
	if (EqList->lstlen == 0) {		// Database empty
		PGAmax = ZIP;
		PGVmax = ZIP;
		MMImax = ONE;
	}

	// Find color of bar.
	ShakeMapColor(&SMCred, &SMCgreen, &SMCblue, MMImax);


	// Find size of bar
	left = BarX;
	right = BarX + BarW;
	bottom = (int)(ScaleY + ScaleH);
	top = ScaleSplitMMI(MIN(MMImax, THRESH_MMI_CLIP));


	// Draw MMI histogram bar
	hBrush = CreateSolidBrush(RGB(SMCred, SMCgreen, SMCblue));
	SelectObject(hdc, hBrush);
	SetRect(&rctngl, left, top, right, bottom);
	FillRect(hdc, &rctngl, hBrush);
	DeleteObject(hBrush);

	// Label bargraph portion "Intensity at <this Site>" or "No Earthquake Shown".
	// (Lable disappears above MMI 9.5 for esthetic reasons)
	SetTextColor(hdc, BLACK);
	SelectObject(hdc, hFontTR18b);
	SetTextAlign(hdc, TA_CENTER);

	if (IsCurrentEq && (MMImax <= 9.5)) {
		top = ScaleSplitMMI(THRESH_MMI_CLIP) -  LineHeightTR18b / 9;
		topb = ScaleSplitMMI(MMImax) - LineHeightTR18b;
		top = MAX(top, topb);
		sprintf(msgstr, "Intensity at %s", Site.StationSite);
		TextOut(hdc, ((left + right) / 2), top, (LPCTSTR)msgstr, strlen(msgstr));
	}

	if (EqList->lstlen == 0) {			// Database empty
		IsCurrentEq = FALSE;
		left = (int)BarX;
		right = (int)(BarX + BarW);
		top = ScaleSplitMMI((double)6.0) -  LineHeightTR18b / 2;
		TextOut(hdc, ((left + right) / 2), top, (LPCTSTR)"No Earthquake Shown", 19);
	}


	// Return TRUE if on scale, FALSE if clipped.
	if (MMImax > THRESH_MMI_CLIP)
		return (FALSE);
	else
		return (TRUE);

}	// DrawMMIBar()


////////////////////////////////////////////////////////////////////////////
// Draw the scale bar and its lables, and lay out space for the histogram
// bar for Instrumental Intensity.  Also draw USGS logo.
//
// The centers of the color scale map to integer intensity values as follows:
//
//  |   I   | II-III|   IV  |   V   |   VI  |  VII  |  VIII |   IX  |   X+  |
//      1     2   3     4       5       6       7       8       9       10
//
// however, in this case, since MMI is undefined below 1.0 and saturated above
// 10.0, and because the nonlinear treatment of II and III is very inconvenient
// here and squeezes more routine action into a narrower range, I am making the
// scale linear from 1.0 to 10.0:
//
//  I   |   II  |  III  |   IV  |   V   |   VI  |  VII  |  VIII |   IX  |   X+
//  1       2       3       4       5       6       7       8       9       10
//  |   |       |       |       |       |       |       |       |       |   |
//

// ***** process *****
void
DrawScalesLables(hdc, MMIhist)
	HDC			hdc;
	double		MMIhist;		// Intensity of corresponding histogram bar (zero if none)
{
	HBRUSH		hBrush;			// Brush
	POINT		pt;				// Current point

	RECT		rctngl;			// Rectangle
	int			left, top, right, bottom, labelYdiff, ixx, iyy, ixxr, iybias;
	int			ic, SMCred, SMCgreen, SMCblue;
	double		Dtop, Dbottom, ScaleInc, ScaleIncY, MMImiddle;
	int			MMIint;


	// ----------------------------------------------------------------------------------
	// Line between controll and histogram
	SelectObject(hdc, GetStockObject(BLACK_PEN));

	MoveToEx(hdc, (int)(BarX - HALF_GUTTER), (int)IntTtlY, &pt);
	LineTo(hdc, (int)(BarX - HALF_GUTTER), (int)(BarY + BarH));
	// Line between scale bar and histogram
	MoveToEx(hdc, (int)(ScaleX - HALF_GUTTER), (int)BarY, &pt);
	LineTo(hdc, (int)(ScaleX - HALF_GUTTER), (int)(BarY + BarH));


	// ----------------------------------------------------------------------------------
	// Draw color scale bar ...
	left  = (int)ScaleX;
	right = (int)(ScaleX + ScaleW);

	// ... first slice ...
	ScaleInc  = (double)9.0 / (double)N_COLOR_SLICES;				// MMI/sclice
	MMImiddle = ONE;												// start in middle of MMI=I

	ScaleIncY = ScaleH / (double)N_COLOR_SLICES;					// pixels/slice
	Dbottom = ScaleY + ScaleH;
	bottom = (int)(Dbottom + HALF);
	Dtop = Dbottom - ScaleIncY;										// Y is positive down
	top = (int)(Dtop + HALF);

	// ... remaining slices ...
	for (ic = 0 ; ic < N_COLOR_SLICES ; ic++) {

		// Draw last slice.
		ShakeMapColor(&SMCred, &SMCgreen, &SMCblue, MMImiddle);

		hBrush = CreateSolidBrush(RGB(SMCred, SMCgreen, SMCblue));
		SelectObject(hdc, hBrush);
		SetRect(&rctngl, left, top, right, bottom);
		FillRect(hdc, &rctngl, hBrush);
		DeleteObject(hBrush);

		// Compute next slice.
		Dbottom = Dtop;
		bottom = top;
		Dtop = Dbottom - ScaleIncY;									// Y is positive down
		top = (int)(Dtop + HALF);
		MMImiddle = MMImiddle + ScaleInc;
	}

	// ... black outline ...
	DeleteObject(SelectObject(hdc, GetStockObject(NULL_BRUSH)));
	top = (int)ScaleY;
	bottom = (int)(ScaleY + ScaleH + HALF);
	Rectangle(hdc, left, top, right, bottom + 1);	// Fudge bottom by one pixel to overcome MS oddity

	// ----------------------------------------------------------------------------------
	// ... partition lines and labels ...
	SelectObject(hdc, hFontTR12b);
	SetTextColor(hdc, BLACK);
	SetTextAlign(hdc, TA_CENTER);

	// ... ("top" is the MMI (0.5) bound; labels are offset to centers (0.0) by "labelYdiff") ...
	ScaleIncY = (double)ScaleH / (double)9.0;						// pixels/slice
	Dtop = (double)(ScaleY + ScaleH) - (ScaleIncY / TWO);			// Y is positive down
	top = (int)(Dtop + HALF);										// Top of MMI = I
	labelYdiff = (int)((ScaleIncY / TWO) + HALF);

	ixx = (left + right) / 2;
	iyy = (int)(HALF + Dtop) + labelYdiff - LineHeightTR12b / 2;

	SelectObject(hdc, hPenSolid1pt);
	MoveToEx(hdc, (int)BarX,          (int)(ScaleY + ScaleH), &pt);
	LineTo(hdc,   (int)(BarX + BarW), (int)(ScaleY + ScaleH));

	SetBkMode(hdc, OPAQUE);
	for (MMIint = 1 ; MMIint <= 9 ; MMIint++) {						// Note FORTRAN style index

		// Text in color bar
		if (MMIint == 9)
			SetTextColor(hdc, WHITE);

		TextOut(hdc, ixx, iyy - 2,
			(LPCTSTR)MMIlabels[MMIint - 1],
			strlen(MMIlabels[MMIint - 1]));

		// (opaque only for MMIint=I and X+)
		SetBkMode(hdc, TRANSPARENT);

		// Dotted lines in histogram bar.
		if ((MMIhist < (double)8.5) || (((double)MMIint + HALF) > MMIhist)) {
			SelectObject(hdc, hPenDottedBlack1pt);
		}
		else {
			SelectObject(hdc, hPenDottedWhite1pt);					// White only for dark red bars
		}

		MoveToEx(hdc, (int)BarX,          top, &pt);				// Line below each MMI
		LineTo(hdc,   (int)(BarX + BarW), top);

		// Update for next Intensity
		Dtop = Dtop - ScaleIncY;
		top = (int)(Dtop + HALF);
		iyy = top + labelYdiff - LineHeightTR12b / 2;
	}

	// Don't forget "X+"
	SetBkMode(hdc, OPAQUE);
	SetTextColor(hdc, BLACK);
	TextOut(hdc, ixx, iyy + 1,
		(LPCTSTR)MMIlabels[MMIint - 1],
		strlen(MMIlabels[MMIint - 1]));
	SelectObject(hdc, hPenSolid1pt);
	MoveToEx(hdc, (int)BarX,          (int)ScaleY, &pt);
	LineTo(hdc,   (int)(BarX + BarW), (int)ScaleY);


	// ----------------------------------------------------------------------------------
	// Framework for legend.
	left = PrcvdShkX;				// Left vertical
	top = IntTtlY;
	bottom = PrcvdShkY + PrcvdShkH;
	MoveToEx(hdc, left, top, &pt);
	LineTo(hdc, left, bottom);

	left = PotDmgX;					// Center vertical
	top = IntTtlY;
	bottom = PotDmgY + PotDmgH;
	MoveToEx(hdc, left, top, &pt);
	LineTo(hdc, left, bottom);

	right = PotDmgX + PotDmgW;		// Right vertical
	top = IntTtlY;
	bottom = PotDmgY + PotDmgH;
	MoveToEx(hdc, right, top, &pt);
	LineTo(hdc, right, bottom);


	// ... (top is MMI (0.5) bound; labels are offset to (0.0) centers by "labelYdiff") ...
	ScaleIncY = (double)ScaleH / (double)9.0;						// pixels/slice
	Dtop = (double)(ScaleY + ScaleH) - (ScaleIncY / TWO);			// Y is positive down
	top = (int)(Dtop + HALF);

	SelectObject(hdc, hPenSolid1pt);
	MoveToEx(hdc, (int)PrcvdShkX,          (int)(PotDmgY + PotDmgH), &pt);
	LineTo(hdc,   (int)(PotDmgX + PotDmgW),(int)(PotDmgY + PotDmgH));

	for (MMIint = 1 ; MMIint <= 9 ; MMIint++) {						// Note FORTRAN style index

		MoveToEx(hdc, (int)PrcvdShkX,           top, &pt);			// Line above each MMI
		LineTo(hdc,   (int)(PotDmgX + PotDmgW), top);

		// Update for next Intensity
		Dtop = Dtop - ScaleIncY;									// Y is positive down
		top = (int)(Dtop + HALF);
	}

	// Don't forget "X+"
	MoveToEx(hdc, (int)PrcvdShkX,           (int)PotDmgY, &pt);
	LineTo(hdc,   (int)(PotDmgX + PotDmgW), (int)PotDmgY);


	// ----------------------------------------------------------------------------------
	// Intensity title spaces

	SelectObject(hdc, hFontTR22b);
	SetTextColor(hdc, BLACK);
	SetTextAlign(hdc, TA_CENTER);
	left = IntTtlX;
	right = ScaleX + ScaleW;
	ixx = (left + right) / 2;
	bottom = 3 + (IntTtlH - LineHeightTR22b) / 2;
	sprintf(msgstr, "Intensity at %s", Site.StationSite);
	TextOut(hdc, ixx, bottom, (LPCTSTR)msgstr, strlen(msgstr));

	SelectObject(hdc, hFontTR12b);
	left = PrcvdShkX;
	right = PrcvdShkX + PrcvdShkW;
	ixx = (left + right) / 2;
	top = IntTtlY;
	bottom = top + LineHeightTR12b;
	TextOut(hdc, ixx, top, (LPCTSTR)"Perceived", 9);
	TextOut(hdc, ixx, bottom, (LPCTSTR)"Shaking", 7);

	left = PotDmgX;
	right = PotDmgX + PotDmgW;
	ixx = (left + right) / 2;
	top = IntTtlY;
	bottom = top + LineHeightTR12b;
	TextOut(hdc, ixx, top, (LPCTSTR)"Potential", 9);
	TextOut(hdc, ixx, bottom, (LPCTSTR)"Damage", 6);


	// ----------------------------------------------------------------------------------
	// Lable with perceived shaking and potential damage

	left  = (int)PrcvdShkX;
	right = (int)(PrcvdShkX + PrcvdShkW);
	ixx = (left + right) / 2;

	left  = (int)PotDmgX;
	right = (int)(PotDmgX + PotDmgW);
	ixxr = (left + right) / 2;

	Dtop = (double)(PrcvdShkY + PrcvdShkH) - (ScaleIncY / TWO);
	iyy = (int)(HALF + Dtop) + labelYdiff - LineHeightTR12 / 2;

	SelectObject(hdc, hFontTR12);
	SetTextColor(hdc, BLACK);
	SetTextAlign(hdc, TA_CENTER);
	SetBkMode(hdc, TRANSPARENT);

	for (MMIint = 1 ; MMIint <= 9 ; MMIint++) {						// Note FORTRAN style index

		if (MMIint == 1)
			iybias = (int)(HALF + ScaleIncY / (double)4);
		else
			iybias = 0;

		MMIphrases((double)MMIint, PercvdShake, PotDmg);
		TextOut(hdc, ixx, iyy - iybias, (LPCTSTR)PercvdShake, strlen(PercvdShake));
		if (MMIint != 8) {
			TextOut(hdc, ixxr, iyy - iybias, (LPCTSTR)PotDmg, strlen(PotDmg));
		}
		else {
			TextOut(hdc, ixxr, iyy - (LineHeightTR12 / 2), (LPCTSTR)"Moderate", 8);
			TextOut(hdc, ixxr, iyy + (LineHeightTR12 / 2), (LPCTSTR)"/Heavy", 6);
		}

		// Update for next Intensity
		Dtop = Dtop - ScaleIncY;								// Y is positive down
		iyy = (int)(HALF + Dtop) + labelYdiff - LineHeightTR12b / 2;
	}

	// Don't forget "X+"
	MMIphrases((double)MMIint, PercvdShake, PotDmg);
	iybias = (int)(HALF + ScaleIncY / (double)4);
	TextOut(hdc, ixx, iyy + iybias, (LPCTSTR)PercvdShake, strlen(PercvdShake));
	TextOut(hdc, ixxr, iyy + iybias, (LPCTSTR)PotDmg, strlen(PotDmg));

}	//DrawScalesLables()


////////////////////////////////////////////////////////////////////////////
// Convert epochal time to a date-time string (fits into [MEDIUM_STR]).

// ***** process *****
char *
EpochToStr(double epoch)
{
		int		yr, mo, dy, hr, mn;
		double	sec;
static	char	TimeStr[MEDIUM_STR];

	datime(epoch, &yr, &mo, &dy, &hr, &mn, &sec, GREGORIAN);
	sprintf(TimeStr, "%02d/%02d/%04d  %02d:%02d:%02.0lf",
		mo, dy, yr, hr, mn, sec);

	return(TimeStr);
}	// EpochToStr()


////////////////////////////////////////////////////////////////////////////
// General error exit--post message, get "OK", then destroy process.
// (Newlines need only be "\n" for ErrExit().)

// ***** process *****
void
ErrExit(char * msg)
{
	MessageBox(ghWndMain, msg, "SideBar()", MB_ICONEXCLAMATION | MB_OK);

	(void)PostMessage(ghWndMain, WM_DESTROY, (WPARAM)0, (LPARAM)0);
}	// ErrExit()


//////////////////////////////////////////////////////////////////////////////
// Add new warnign message to the warnings diplay buffer.  Display updated
// buffer in warnings edit control.  (Newlines need to be "\r\n" for
// ErrWarn() and each message sent to it should terminate with "\r\n".)

// ***** process *****
void
ErrWarn(char * msg)
{
	int need, have, shiftby, iWarn;

	/* Make space in Site.Warn ... */
	need = (int)strlen(msg);
	have = WARN_BUF_SIZE - 1 - Site.WarnSize;	// "- 1" for the terminating null
	shiftby = MAX(0, need - have);

	if (shiftby > 0) {
		Site.WarnSize -= shiftby;
		for (iWarn = 0 ; iWarn < Site.WarnSize ; iWarn++) {
			Site.Warn[iWarn] = Site.Warn[iWarn + shiftby];
		}
		Site.Warn[iWarn] = '\0';
	}

	/* ... and put the new packet into it ... */
	strcat(Site.Warn, msg);
	Site.WarnSize = (int)strlen(Site.Warn);

	// ... attach Warnings buffer to window ...
	SetWindowText(hWndMsgWarn, Site.Warn);

	// ... and bring it to attention by scrolling down to it.
	SendMessage(hWndMsgWarn, EM_LINESCROLL, (WPARAM)0, (LPARAM)(WARN_BUF_SIZE / 2));
}	// ErrWarn()


////////////////////////////////////////////////////////////////////////////
// Remove a full line from FIFO, or return NULL.

// ***** process *****
char *
FIFOgets(int * nFIFO, char * FIFO)
{
static	char	outbuf[BUF_SIZE];
		char	*retn;
		char	*lbrk;
		int		iFIFO, jj, kk;

	// Sanity check
	if (*nFIFO <= 0) {
		FIFO[0] = '\0';
		return(NULL);
	}
	outbuf[0] = '\0';

	// Skip over leading returns
	iFIFO = 0;
	while (FIFO[iFIFO] == '\n' || FIFO[iFIFO] == '\r') {
		iFIFO++;
	}

	// Check if a full line is available
	lbrk = strpbrk(FIFO + iFIFO, "\n\r");

	if (lbrk == NULL) {
		retn = NULL;
	}

	else {
		// Copy over one line.
		jj = 0;
		while (FIFO[iFIFO] != '\n' && FIFO[iFIFO] != '\r') {
			outbuf[jj] = FIFO[iFIFO];
			iFIFO++;
			jj++;
		}
		outbuf[jj] = '\0';
		retn = outbuf;
	}

	// Clean that line (and/or leading returns) out of the FIFO.
	// (This algorithm may leave a residual of '\r' and/or '\n'
	// characters at beginning of the FIFO.)
	if (iFIFO > 0) {

		kk = 0;
		for (jj = iFIFO ; jj < *nFIFO ; jj++) {
			FIFO[kk] = FIFO[jj];
			kk++;
		}
		FIFO[kk] = '\0';
		*nFIFO = kk;
	}

	return(retn);
}	// FIFOgets()


////////////////////////////////////////////////////////////////////////////
// Add bytes from NewBytes to FIFO.  Keep FIFO null-byte terminated (hence,
// there is room for (maxFIFO - 1) characters plus the terminating null).
// Return TRUE if they all fit, FALSE if not (extra bytes are discarded).
// nFIFO is the number of bytes excluding the terminating null, so it will
// never exceed (maxFIFO - 1).  NOTE:  Blanks out NewBytes upon return.

// ***** process *****
BOOL
FIFOstuff(int * nFIFO, int maxFIFO, char * FIFO,
		  char * NewBytes, int nNewBytes)
{
	int		ii;

	// Immediate returns
	if (nNewBytes == 0) {			// Nothing to add
		NewBytes[0] = '\0';
		return(TRUE);
	}

	if (*nFIFO == (maxFIFO - 1)) {	// No room at the Inn
		NewBytes[0] = '\0';
		nNewBytes = 0;
		return(FALSE);
	}

	// Error checks
	if (*nFIFO < 0) {
		ErrWarn("nFIFO <0 in FIFOstuff().\r\n  Reset to zero.\r\n");
		*nFIFO = 0;
	}
	if (*nFIFO > (maxFIFO - 1)) {
		ErrWarn("nFIFO >(maxFIFO - 1) in FIFOstuff().\r\n  Reset to (maxFIFO - 1).\r\n");
		*nFIFO = maxFIFO - 1;
	}
	if (nNewBytes < 0) {
		ErrWarn("nNewBytes <0 in FIFOstuff().\r\n.");
	}

	// Stuff into FIFO
	ii = 0;
	while (ii < nNewBytes) {
		FIFO[*nFIFO] = NewBytes[ii];
		(*nFIFO)++;
		ii++;

		// Partial success, but ran out of FIFO space
		if (ii < nNewBytes && *nFIFO >= (maxFIFO - 1)) {

			FIFO[maxFIFO - 1] = '\0';
			*nFIFO = maxFIFO - 1;

			NewBytes[0] = '\0';
			nNewBytes = 0;

			return(FALSE);
		}
	}

	// Success
	FIFO[*nFIFO] = '\0';
	NewBytes[0] = '\0';
	nNewBytes = 0;
	return(TRUE);

}	// FIFOstuff()


////////////////////////////////////////////////////////////////////////////
// Tag the N2PROTECT largest-Intensity events to protect them from deletion.

// ***** process *****
void
FindLargestEvents()
{
	EqEntry			*TempEq;			// Earthquake to consider.
	double			*MMItemp, MMItmp, TopMMIs[N2PROTECT];
	int				ii, jj;

	if (EqList->lstlen == 0) {			// Database empty
		GetCurrentEq();					// Sets flags right
		return;
	}


	// Create array space for list of MMIs
	MMItemp = (double *)malloc((size_t)(sizeof(double) * (EqList->lstlen)));

	// First pass through list:  Make a list of MMIs,
	// and initialize "largest MMI" flags.
	ll_head(EqList);					// Null earthquake (list head is always empty)
	ii = 0;
	while (ll_next(EqList)) {
		TempEq = (EqEntry *)ll_ret(EqList);
		*(MMItemp + ii) = TempEq->MMI;
		ii++;
		TempEq->LargestMMIs = FALSE;
	}


	// Second pass (only through list of MMIs):  Find the magnitudes
	// of the three largest MMIs.  Destroys list.
	for (jj = 0 ; jj < 3 ; jj++) {

		TopMMIs[jj] = ZIP;
		for (ii = 0 ; ii < EqList->lstlen ; ii++) {

			// Kill off any previous winners by zeroing them (i.e.,
			// effectively remove all previous maxima from MMI list)
			if (jj > 0 && *(MMItemp + ii) == TopMMIs[jj-1]) {
				*(MMItemp + ii) = ZIP;
			}

			// Find the next largest MMI
			TopMMIs[jj] = MAX(TopMMIs[jj], *(MMItemp + ii));
		}
	}


	// Third pass through list:  Mark as "largest" all those events
	// matching one of the N2PROTECT largest MMIs.
	ll_head(EqList);					// Null earthquake (list head is always empty)
	while (ll_next(EqList)) {
		TempEq = (EqEntry *)ll_ret(EqList);
		MMItmp = TempEq->MMI;

		if (CheckMMImatches(MMItmp, TopMMIs)) {
			TempEq->LargestMMIs = TRUE;
		}
	}


	// Reposition database back to current earthquake
	if (IsCurrentEq) {
		GetCurrentEq();
	}

}	// FindLargestEvents()


////////////////////////////////////////////////////////////////////////////
// Retrieve the current, displayable earthquake.  Sets IsCurrentEq = TRUE
// when successful, FALSE when not.  Will reset ShowEqType if necessary to
// succeed.

// ***** process *****
void
GetCurrentEq()
{
	double		PGAtemp, PGVtemp;


	// No earthquake is available:
	// return the fact that nothing is displayable
	// and set reasonable NULL values for key variables,
	// including ShowEqType to its default.
	if (EqList->lstlen == 0) {
		CurrentEq = NULL;
		IsCurrentEq = FALSE;
		ShowEqType = SHOW_RECENT;

		PGAmax = ZIP;
		PGVmax = ZIP;
		MMImax = ONE;
		return;
	}


	// Move linked list pointer to correct earthquake
	if (ShowEqType == SHOW_ANCIENT) {
		ll_head(EqList);	// oldest is stored next-to-first (first is null)
		(void)ll_next(EqList);
	}

	else if (ShowEqType == SHOW_RECENT) {
		ll_tail(EqList);	// latest is stored last
	}
	
	else if (ShowEqType == SHOW_BIGGEST) {

		// Find the largest Instrumental Intensity
		ll_head(EqList);		// Null earthquake
		MMImax = ZIP;

		while (ll_next(EqList)) {
			CurrentEq = (EqEntry *)ll_ret(EqList);
			PGAtemp = MAX3(CurrentEq->PGAch1, CurrentEq->PGAch2, CurrentEq->PGAch3);
			PGVtemp = MAX3(CurrentEq->PGVch1, CurrentEq->PGVch2, CurrentEq->PGVch3);
			MMImax  = MAX(MMImax, CurrentEq->MMI);
		}

		// Then find the MOST RECENT earthquake matching this MMI.
		ll_tail(EqList);
		CurrentEq = (EqEntry *)ll_ret(EqList);
		while (!MatchSize()) {	// MatchSize() sets (PGAmax, PGVmax, MMImax) when it matches.
			if (!ll_prev(EqList)) {
				ErrWarn("Ran off bottom of linked list\r\n  while looking for biggest.\r\n  Defaulting to latest.\r\n");
				ShowEqType = SHOW_RECENT;
				HighlightEventButton();
				ll_tail(EqList);
				break;
			}
			CurrentEq = (EqEntry *)ll_ret(EqList);
		}
	}
	
	else if (ShowEqType == SHOW_SELECTED_P || ShowEqType == SHOW_SELECTED_N) {

		if ((!IsCurrentEq) || (CurrentEq == NULL)) {
			sprintf(msgstr,
				"Bug in GetCurrentEq():\nNo CurrentEq.\nEqList->lstlen = %d.",
				EqList->lstlen);	// If lstlen == 0, default as in opening lines of routine.
			ErrExit(msgstr);
			ErrExit("Bug in GetCurrentEq():\nNo CurrentEq.");
		}

		ll_head(EqList);			// Reposition disturbed database
		if (FAIL == ll_find(EqList, (char *)CurrentEq, MatchEvent)) {
			ll_tail(EqList);		// default to most recent event
			ShowEqType = SHOW_RECENT;
		}
	}	// end ShowEqType event search


	// Now go get that earthquake, and note its maxima [it CAN be deleted
	// in the meantime by other earthquakes, hence the if()]
	CurrentEq = (EqEntry *)ll_ret(EqList);
	if (NULL != CurrentEq) {
		PGAmax = MAX3(CurrentEq->PGAch1, CurrentEq->PGAch2, CurrentEq->PGAch3);
		PGVmax = MAX3(CurrentEq->PGVch1, CurrentEq->PGVch2, CurrentEq->PGVch3);
		MMImax = PGAVtoMMI(((PGAmax / HH) * ONE_G ), PGVmax);
		IsCurrentEq = TRUE;
	}
	
	else {
		IsCurrentEq = FALSE;
	}


	// Find the position of this event in the linked list
	if (ListPositionOK && (EqList->lstlen > 0)) {			// Database has data
		iListPosition = ListPosition();
	}
	else {
		iListPosition = 0;
	}


	// If failed and not the best try, recurse with a better try
	if (	(!IsCurrentEq) &&
			(ShowEqType != SHOW_RECENT) &&
			(EqList->lstlen != 0)
		) {

		ShowEqType = SHOW_RECENT;
		GetCurrentEq();
	}


	// Done
	return;

}	// GetCurrentEq()


////////////////////////////////////////////////////////////////////////////
// Read size of main window and viewport, and origin point, and infer
// caption bar width.

// ***** process *****

void
GetWindowLayout(HWND hwnd, WinLayout * Layout)
{
	RECT	Wsize;
	RECT	Vsize;
	BOOL	RetVal;

	RetVal = GetWindowRect(hwnd, (LPRECT)&Wsize);
	Layout->WinSizeX = Wsize.right - Wsize.left;
	Layout->WinSizeY = Wsize.bottom - Wsize.top;

	RetVal = GetClientRect(hwnd, (LPRECT)&Vsize);
	Layout->ViewSizeX = Vsize.right - Vsize.left;
	Layout->ViewSizeY = Vsize.bottom - Vsize.top;

	Layout->WndBdr = (Layout->WinSizeX - Layout->ViewSizeX) / 2;
	Layout->CaptionH = Layout->WinSizeY - Layout->ViewSizeY - (2 * Layout->WndBdr);
}


////////////////////////////////////////////////////////////////////////////
// Grab the real-time (trigger) messages from data stream, compensate them,
// add them to data base, and signal a display update.  If it runs into
// trouble (from VerboseTRG() not recognizing a format, for example, it
// simply bails out without doing anythingz).
//
// CAVEAT:  Input to GrabTrg() is BY THE LINE, not by UDP packets, so
// multiple calls span one trigger event and some faith that nothing gets
// lost is implicit.  When something is lost midway in a trigger message,
// the results are unpredictable, but may include mixing of data from
// adjoining trigger events, particularly because every other event is
// ignored (KeepToggle) to avoid event duplication.

// ***** process *****
void
GrabTrg(char * Line)
{
		char		*VerBuf;				// Verbose text buffer
		char		*chVer;					// Pointer to current position in VerBuf

static	char		TriggerTime[LONG_STR];	// String with time of trigger
static	double		Ch1PGA, Ch2PGA, Ch3PGA;	// Metrics
static	double		Ch1PGV, Ch2PGV, Ch3PGV;
static	double		Ch1PGVt, Ch2PGVt, Ch3PGVt;
static	double		P1, P1Ch2Sa, P1Ch3Sa;
static	double		P2, P2Ch2Sa, P2Ch3Sa;
static	double		P3, P3Ch2Sa, P3Ch3Sa;
static	double		Temperature;

static	int			TRGline;				// To check message continuity
											// and switch extraction code.
static	BOOL		Continuity;

		EqEntry		*Evt;
		SYSTEMTIME	LocalTime;

	// Process only lines that contain triggers
	if (NULL == strstr(Line, ":TRG"))
		return;

	// If start of a new trigger message, start a new data-base
	// (linked list) item, and note the message arrival time.
	//
	// CAVEAT:  This is the ONE AND ONLY "official" earthquake time
	// as seen by SideBar(), and is controlled ONLY by how well the
	// PCs clock is set.  The USER IS RESPONSIBLE for keeping PC
	// reasonably well synched to real time.

	if (NULL != strstr(Line, ":TRG0")) {

		// Start tracking message continuity
		Continuity = TRUE;

		// Start keeping track of message continuity
		TRGline = 0;

		// Note current local time
		GetLocalTime(&LocalTime);
		Site.PktArrTime = timvar(
			(int)LocalTime.wYear,
			(int)LocalTime.wMonth,
			(int)LocalTime.wDay,
			(int)LocalTime.wHour,
			(int)LocalTime.wMinute,
			(double)LocalTime.wSecond + (double)LocalTime.wMilliseconds / (double)1000.,
			GREGORIAN);

		// Adjust back to estimated onset time and translate to ascii.
		Site.PktArrTime -= Site.TimeFudge;
		strcpy(Site.chPktArrTime, EpochToStr(Site.PktArrTime));
	}

	// Check message continuity
	else if (Continuity && NULL != strstr(Line, ":TRG1")) {
		if (TRGline != 0)
			Continuity = FALSE;
		TRGline = 1;
	}
	else if (Continuity && NULL != strstr(Line, ":TRG2")) {
		if (TRGline != 1)
			Continuity = FALSE;
		TRGline = 2;
	}
	else if (Continuity && NULL != strstr(Line, ":TRG3")) {
		if (TRGline != 2)
			Continuity = FALSE;
		TRGline = 3;
	}
	else if (Continuity && NULL != strstr(Line, ":TRGV")) {
		if (TRGline != 3)
			Continuity = FALSE;
		TRGline = 4;
	}
	else if (Continuity && NULL != strstr(Line, ":TRGS")) {	// 5,6,7
		if (TRGline < 4 || TRGline > 6)
			Continuity = FALSE;
		TRGline++;
	}
	else {
		// Process only lines that contain triggers.
		// Reaching this point indicates a programming
		// bug or garbled input.  Ignore subsequent
		// trigger input until next ":TRG0".
		TRGline = 99;
		Continuity = FALSE;
		return;
	}

	// Example of raw trigger message:
	//
	// P13:TRG0;892752629.085;1372
	// P13:TRG1;6061;892752629.845;427
	// P13:TRG2;10160;892752629.835;427
	// P13:TRG3;878;892752629.115;427
	// P13:TRGV;523 550;26080 504;40800 504
	// P13:TRGS;300;429184 655921;6
	// P13:TRGS;1000;434058 696506;9
	// P13:TRGS;3000;492739 714771;12

	// Make the line verbose
	VerBuf = VerboseTRG(Line);

	// CAVEAT:  If line is untranslatable, bail out.  But it may be
	// trouble if one has reaced this spot with an untranslatable line.
	// This can mean that a new event item is open, for example.  So this
	// brute-force abandonment is not necessarily wise.  More likely, one
	// simply has stumbled into the middle of a transmission, for which
	// this may be a perfectly good solution.
	if (VerBuf == NULL)
		return;

	// Example of verbose result:
	//
	// 0 P13  TRIGGERED  (12.56V)
	//             2001/07/30   22:41:10.130 ADC (Corrected by -3696.540 s)
	// 1 P13  Ch1 PGA=  16.38 gal @ 0.31 s  28.34C
	// 2 P13  Ch2 PGA= 733.80 gal @ 0.33 s  28.34C
	// 3 P13  Ch3 PGA= 416.89 gal @ 0.32 s  28.34C
	// 4 P13  PGV Ch1=  0.72 Ch2= 30.90 Ch3= 19.05 cm/s, 28.34C (4.40, 4.24, 4.23 s into)
	// 5 P13  Sa#1 0.30 s  Ch2=1543.35  Ch3=1001.67 cm/s
	// 6 P13  Sa#2 1.00 s  Ch2= 162.37  Ch3= 100.06 cm/s
	// 7 P13  Sa#3 3.00 s  Ch2=  25.20  Ch3=  14.98 cm/s
	//   P13  Saved in file 1 of 2 [codes 0x00000000]

	// Extract the needed bits
	chVer = VerBuf;

	switch (TRGline) {
	case (0):
		chVer = 10 + strstr(VerBuf, "          ");
		strncpy(TriggerTime, chVer, 11);
		strncpy(TriggerTime + 11, chVer + 13, 12);
		TriggerTime[23] = '\0';
		break;
	case (1):
		chVer = 8 + strstr(chVer, "Ch1 PGA=");
		Ch1PGA = HH * atof(chVer) / ONE_G;	// gal --> %g
		chVer = 2 + strstr(chVer, " s");
		Temperature = atof(chVer);
		break;
	case (2):
		chVer = 8 + strstr(chVer, "Ch2 PGA=");
		Ch2PGA = HH * atof(chVer) / ONE_G;	// gal --> %g
		break;
	case (3):
		chVer = 8 + strstr(chVer, "Ch3 PGA=");
		Ch3PGA = HH * atof(chVer) / ONE_G;	// gal --> %g
		break;
	case (4):
		chVer = 8 + strstr(chVer, "PGV Ch1=");
		Ch1PGV = atof(chVer);
		chVer = 4 + strstr(chVer, "Ch2=");
		Ch2PGV = atof(chVer);
		chVer = 4 + strstr(chVer, "Ch3=");
		Ch3PGV = atof(chVer);
		chVer = 3 + strstr(chVer, "C (");
		Ch1PGVt = atof(chVer);
		chVer = 1 + strstr(chVer, ",");
		Ch2PGVt = atof(chVer);
		chVer = 1 + strstr(chVer, ",");
		Ch3PGVt = atof(chVer);
		break;

	// The following not now used, but for the future ...
	case (5):
		chVer = 5 + strstr(chVer, SaNote1);
		P1 = atof(chVer);
		chVer = 4 + strstr(chVer, "Ch2=");
		P1Ch2Sa = atof(chVer);
		chVer = 4 + strstr(chVer, "Ch3=");
		P1Ch3Sa = atof(chVer);
		break;
	case (6):
		chVer = 5 + strstr(chVer, SaNote2);
		P2 = atof(chVer);
		chVer = 4 + strstr(chVer, "Ch2=");
		P2Ch2Sa = atof(chVer);
		chVer = 4 + strstr(chVer, "Ch3=");
		P2Ch3Sa = atof(chVer);
		break;
	case (7):
		chVer = 5 + strstr(chVer, SaNote3);
		P3 = atof(chVer);
		chVer = 4 + strstr(chVer, "Ch2=");
		P3Ch2Sa = atof(chVer);
		chVer = 4 + strstr(chVer, "Ch3=");
		P3Ch3Sa = atof(chVer);
		break;
	default:
		return;	// Just in case (ignore garbling, TRGline = 99)
	}

	// Put new information into database
	if (Continuity && NULL != strstr(Line, ":TRGS") && TRGline == 7) {
		if (KeepToggle) {
			KeepToggle = FALSE;		// Ignore next, duplicate, earthquake

			// Create item, move to end of list, and add item to list
			Evt = CreateEqItem(Site.PktArrTime, Site.chPktArrTime,
				Ch1PGA, Ch2PGA, Ch3PGA, Ch1PGV, Ch2PGV, Ch3PGV, Temperature);
			ll_tail(EqList);		// This moves the event pointer, so ...
			ll_add(EqList, (char *)Evt);

			// Beep and push SideBar() to foreground.
			(void)MessageBeep((UINT)0xFFFFFFFF);
			(void)OpenIcon(ghWndMain);
			(void)SetForegroundWindow(ghWndMain);

			// ... move it back to current (needed for SHOW_SELECTED_*)
			if (CurrentEq != NULL) {
				ll_head(EqList);		// Null earthquake
				if (FAIL == ll_find(EqList, (char *)CurrentEq, MatchEvent)) {
					ll_tail(EqList);	// default to most recent event
				}
			}

			// Resolve and return current, displayable earthquake
			// (If GetCurrentEq() fails, it defaults to "most recent"
			// and trys again.)
			GetCurrentEq();

			// Signal display update since new earthquake has arrived.
			SendMessage(ghWndMain, WM_UPDATE_WIN, (WPARAM)0, (LPARAM)0);

			// And write out updated earthquake database
			(void)WriteDataBase();
			ll_head(EqList);			// Reposition database, disturbed by WriteDataBase() (null event)
			if (FAIL == ll_find(EqList, (char *)CurrentEq, MatchEvent)) {
				ll_tail(EqList);		// default to most recent event
			}
		}

		else {
			KeepToggle = TRUE;			// Having ignored duplicate, keep next earthquake.
		}
	}

	return;
}	// GrabTrg()


////////////////////////////////////////////////////////////////////////////
// Highlight the active button within the five-button group controlling
// which earthquake is displayed.

// ***** process *****
void
HighlightEventButton()
{
	// De-highlight all buttons
	SendMessage(hWndOldest, BM_SETSTYLE,
		(WPARAM)LOWORD(BS_PUSHBUTTON), MAKELPARAM(TRUE, 0));
	SendMessage(hWndOldest, WM_SETFONT, (WPARAM)hFontTR12, MAKELPARAM(TRUE, 0));

	SendMessage(hWndMostRecent, BM_SETSTYLE,
		(WPARAM)LOWORD(BS_PUSHBUTTON), MAKELPARAM(TRUE, 0));
	SendMessage(hWndMostRecent, WM_SETFONT, (WPARAM)hFontTR12, MAKELPARAM(TRUE, 0));

	SendMessage(hWndBiggest, BM_SETSTYLE,
		(WPARAM)LOWORD(BS_PUSHBUTTON), MAKELPARAM(TRUE, 0));
	SendMessage(hWndBiggest, WM_SETFONT, (WPARAM)hFontTR12, MAKELPARAM(TRUE, 0));

	SendMessage(hWndPrev, BM_SETSTYLE,
		(WPARAM)LOWORD(BS_PUSHBUTTON), MAKELPARAM(TRUE, 0));
	SendMessage(hWndPrev, WM_SETFONT, (WPARAM)hFontTR12, MAKELPARAM(TRUE, 0));

	SendMessage(hWndPrevX5, BM_SETSTYLE,
		(WPARAM)LOWORD(BS_PUSHBUTTON), MAKELPARAM(TRUE, 0));
	SendMessage(hWndPrevX5, WM_SETFONT, (WPARAM)hFontTR12, MAKELPARAM(TRUE, 0));

	SendMessage(hWndNext, BM_SETSTYLE,
		(WPARAM)LOWORD(BS_PUSHBUTTON), MAKELPARAM(TRUE, 0));
	SendMessage(hWndNext, WM_SETFONT, (WPARAM)hFontTR12, MAKELPARAM(TRUE, 0));

	SendMessage(hWndNextX5, BM_SETSTYLE,
		(WPARAM)LOWORD(BS_PUSHBUTTON), MAKELPARAM(TRUE, 0));
	SendMessage(hWndNextX5, WM_SETFONT, (WPARAM)hFontTR12, MAKELPARAM(TRUE, 0));

	// Highlight the one active button (the "X5" buttons never highlight)
	if      (ShowEqType == SHOW_ANCIENT) {
		SendMessage(hWndOldest, BM_SETSTYLE,
			(WPARAM)LOWORD(BS_DEFPUSHBUTTON), MAKELPARAM(TRUE, 0));
		SendMessage(hWndOldest, WM_SETFONT, (WPARAM)hFontTR12b, MAKELPARAM(TRUE, 0));
	}
	else if (ShowEqType == SHOW_RECENT) {
		SendMessage(hWndMostRecent, BM_SETSTYLE,
			(WPARAM)LOWORD(BS_DEFPUSHBUTTON), MAKELPARAM(TRUE, 0));
		SendMessage(hWndMostRecent, WM_SETFONT, (WPARAM)hFontTR12b, MAKELPARAM(TRUE, 0));
	}
	else if (ShowEqType == SHOW_BIGGEST) {
		SendMessage(hWndBiggest, BM_SETSTYLE,
			(WPARAM)LOWORD(BS_DEFPUSHBUTTON), MAKELPARAM(TRUE, 0));
		SendMessage(hWndBiggest, WM_SETFONT, (WPARAM)hFontTR12b, MAKELPARAM(TRUE, 0));
	}
	else if (ShowEqType == SHOW_SELECTED_P) {
		SendMessage(hWndPrev, BM_SETSTYLE,
			(WPARAM)LOWORD(BS_DEFPUSHBUTTON), MAKELPARAM(TRUE, 0));
		SendMessage(hWndPrev, WM_SETFONT, (WPARAM)hFontTR12b, MAKELPARAM(TRUE, 0));
	}
	else if (ShowEqType == SHOW_SELECTED_N) {
		SendMessage(hWndNext, BM_SETSTYLE,
			(WPARAM)LOWORD(BS_DEFPUSHBUTTON), MAKELPARAM(TRUE, 0));
		SendMessage(hWndNext, WM_SETFONT, (WPARAM)hFontTR12b, MAKELPARAM(TRUE, 0));
	}
}	// HighlightEventButton()


////////////////////////////////////////////////////////////////////////////
// Highlight the active button within the three-button group controlling
// which measurement type is displayed.

// ***** process *****
void
HighlightMeasureButton()
{
	// De-highlight all buttons
	SendMessage(hWndPGVslctIn, BM_SETSTYLE,
		(WPARAM)LOWORD(BS_PUSHBUTTON), MAKELPARAM(TRUE, 0));
	SendMessage(hWndPGVslctIn, WM_SETFONT, (WPARAM)hFontTR12, MAKELPARAM(TRUE, 0));

	SendMessage(hWndPGVslctCm, BM_SETSTYLE,
		(WPARAM)LOWORD(BS_PUSHBUTTON), MAKELPARAM(TRUE, 0));
	SendMessage(hWndPGVslctCm, WM_SETFONT, (WPARAM)hFontTR12, MAKELPARAM(TRUE, 0));

	// Highlight the one active button
	if (PGVscale == INCHES_PER_S) {
		SendMessage(hWndPGVslctIn, BM_SETSTYLE,
			(WPARAM)LOWORD(BS_DEFPUSHBUTTON), MAKELPARAM(TRUE, 0));
		SendMessage(hWndPGVslctIn, WM_SETFONT, (WPARAM)hFontTR12b, MAKELPARAM(TRUE, 0));
	}
	else if (PGVscale == CM_PER_S) {
		SendMessage(hWndPGVslctCm, BM_SETSTYLE,
			(WPARAM)LOWORD(BS_DEFPUSHBUTTON), MAKELPARAM(TRUE, 0));
		SendMessage(hWndPGVslctCm, WM_SETFONT, (WPARAM)hFontTR12b, MAKELPARAM(TRUE, 0));
	}
}	// HighlightMeasureButton()


////////////////////////////////////////////////////////////////////////////
// Find the numerical position of the current earthquake within the linked
// list.  Return 0 on failure.

// ***** process *****
int
ListPosition()
{
	EqEntry		*TempEq;
	int			ii;

	if (!ListPositionOK)
		return(0);

	ll_head(EqList);			// Null earthquake
	ii = 0;

	while (ll_next(EqList)) {
		if (!ListPositionOK)	// Try to catch new WM_DESTROY and exit in time
			return(0);

		TempEq = (EqEntry *)ll_ret(EqList);
		ii++;
		if (CurrentEq == TempEq)
			return(ii);
	}

	return(0);
}	// ListPosition()


////////////////////////////////////////////////////////////////////////////
// Populate the control panel.  Controls() responds to their inputs.

// ***** process *****
void
MakeControls(HINSTANCE hInstance)
{
	hWndLogo = CreateWindow("static", LOGO_FILENAME,
		WS_CHILD | WS_VISIBLE | SS_BITMAP,
		LogoX, LogoY,
		LogoW, LogoH,
		ghWndMain, (HMENU)IDM_LOGOS, hInstance, NULL);

	// DrawScale[AV]() actually draws the Logo.
	hLogo = LoadBitmap((HINSTANCE)hInstance, MAKEINTRESOURCE(LOGO_IDint));


	hWndMetricEtc = CreateWindow("edit", NULL,
		WS_CHILD | WS_VISIBLE | WS_BORDER |
		ES_LEFT | ES_READONLY | ES_MULTILINE,
		MetricEtcX, MetricEtcY,
		MetricEtcW, MetricEtcH,
		ghWndMain, (HMENU)IDE_METRIC, hInstance, NULL);
	SendMessage(hWndMetricEtc, WM_SETFONT, (WPARAM)hFontTR18b, MAKELPARAM(FALSE, 0));

	hWndMaxima = CreateWindow("edit", NULL,
		WS_CHILD | WS_VISIBLE | WS_BORDER |
		ES_CENTER | ES_MULTILINE | ES_READONLY |
		ES_AUTOHSCROLL | ES_AUTOVSCROLL,
		MaximaX, MaximaY,
		MaximaW, MaximaH,
		ghWndMain, (HMENU)IDE_SITE_INFO, hInstance, NULL);
	SendMessage(hWndMaxima, WM_SETFONT, (WPARAM)hFontTR18b, MAKELPARAM(FALSE, 0));


	hWndDateTime = CreateWindow("edit", NULL,
		WS_CHILD | WS_VISIBLE | WS_BORDER |
		ES_CENTER | ES_READONLY,
		DateTimeX, DateTimeY,
		DateTimeW, DateTimeH,
		ghWndMain, (HMENU)IDE_DATE_TIME, hInstance, NULL);
	SendMessage(hWndDateTime, WM_SETFONT, (WPARAM)hFontTR18b, MAKELPARAM(FALSE, 0));

	hWndWhichOf = CreateWindow("edit", NULL,
		WS_CHILD | WS_VISIBLE | WS_BORDER |
		ES_CENTER | ES_READONLY,
		WhichOfX, WhichOfY,
		WhichOfW, WhichOfH,
		ghWndMain, (HMENU)IDE_WHICH_OF, hInstance, NULL);
	SendMessage(hWndWhichOf, WM_SETFONT, (WPARAM)hFontTR12b, MAKELPARAM(FALSE, 0));

	hWndOldest = CreateWindow("button", "Oldest",
		WS_CHILD | WS_VISIBLE | WS_BORDER |
		BS_PUSHBUTTON | BS_TEXT | BS_CENTER | BS_VCENTER,
		OldestX, OldestY,
		OldestW, OldestH,
		ghWndMain, (HMENU)IDB_OLDEST, hInstance, NULL);
	SendMessage(hWndOldest, WM_SETFONT, (WPARAM)hFontTR12, MAKELPARAM(TRUE, 0));

	// Highlight this button (BS_DEFPUSHBUTTON), which is the default
	// among the event-selection settings
	ShowEqType = SHOW_RECENT;
	hWndMostRecent = CreateWindow("button", "Latest",
		WS_CHILD | WS_VISIBLE | WS_BORDER |
		BS_DEFPUSHBUTTON | BS_TEXT | BS_CENTER | BS_VCENTER,
		MostRecentX, MostRecentY,
		MostRecentW, MostRecentH,
		ghWndMain, (HMENU)IDB_MOST_RECENT, hInstance, NULL);
	SendMessage(hWndMostRecent, WM_SETFONT, (WPARAM)hFontTR12b, MAKELPARAM(TRUE, 0));

	hWndBiggest = CreateWindow("button", "B i g g e s t",
		WS_CHILD | WS_VISIBLE | WS_BORDER |
		BS_PUSHBUTTON | BS_TEXT | BS_CENTER | BS_VCENTER,
		BiggestX, BiggestY,
		BiggestW, BiggestH,
		ghWndMain, (HMENU)IDB_BIGGEST, hInstance, NULL);
	SendMessage(hWndBiggest, WM_SETFONT, (WPARAM)hFontTR12, MAKELPARAM(TRUE, 0));


	hWndPrev = CreateWindow("button", "Previous",
		WS_CHILD | WS_VISIBLE | WS_BORDER |
		BS_PUSHBUTTON | BS_TEXT | BS_CENTER | BS_VCENTER,
		PrevX, PrevY,
		PrevW, PrevH,
		ghWndMain, (HMENU)IDB_PREVIOUS, hInstance, NULL);
	SendMessage(hWndPrev, WM_SETFONT, (WPARAM)hFontTR12, MAKELPARAM(TRUE, 0));

	hWndPrevX5 = CreateWindow("button", "X5",
		WS_CHILD | WS_VISIBLE | WS_BORDER |
		BS_PUSHBUTTON | BS_TEXT | BS_CENTER | BS_VCENTER,
		PrevX5X, PrevX5Y,
		PrevX5W, PrevX5H,
		ghWndMain, (HMENU)IDB_PREVIOUS_X5, hInstance, NULL);
	SendMessage(hWndPrevX5, WM_SETFONT, (WPARAM)hFontTR12, MAKELPARAM(TRUE, 0));


	hWndNext = CreateWindow("button", "Next",
		WS_CHILD | WS_VISIBLE | WS_BORDER |
		BS_PUSHBUTTON | BS_TEXT | BS_CENTER | BS_VCENTER,
		NextX, NextY,
		NextW, NextH,
		ghWndMain, (HMENU)IDB_NEXT, hInstance, NULL);
	SendMessage(hWndNext, WM_SETFONT, (WPARAM)hFontTR12, MAKELPARAM(TRUE, 0));

	hWndNextX5 = CreateWindow("button", "X5",
		WS_CHILD | WS_VISIBLE | WS_BORDER |
		BS_PUSHBUTTON | BS_TEXT | BS_CENTER | BS_VCENTER,
		NextX5X, NextX5Y,
		NextX5W, NextX5H,
		ghWndMain, (HMENU)IDB_NEXT_X5, hInstance, NULL);
	SendMessage(hWndNextX5, WM_SETFONT, (WPARAM)hFontTR12, MAKELPARAM(TRUE, 0));


	hWndSiteInfo = CreateWindow("edit", NULL,
		WS_CHILD | WS_VISIBLE | WS_BORDER |
		ES_CENTER | ES_MULTILINE | ES_READONLY |
		ES_AUTOHSCROLL | ES_AUTOVSCROLL,
		SiteInfoX, SiteInfoY,
		SiteInfoW, SiteInfoH,
		ghWndMain, (HMENU)IDE_SITE_INFO, hInstance, NULL);
	SendMessage(hWndSiteInfo, WM_SETFONT, (WPARAM)hFontTR18b, MAKELPARAM(FALSE, 0));


	// Exit and messages
	hWndQuit = CreateWindow("button", "Quit",
		WS_CHILD | WS_VISIBLE | WS_BORDER |
		BS_PUSHBUTTON | BS_TEXT | BS_CENTER | BS_VCENTER,
		QuitX, QuitY,
		QuitW, QuitH,
		ghWndMain, (HMENU)IDB_QUIT, hInstance, NULL);
	SendMessage(hWndQuit, WM_SETFONT, (WPARAM)hFontTR12, MAKELPARAM(TRUE, 0));

	hWndMsgWarn = CreateWindow("edit", NULL,
		WS_CHILD | WS_VISIBLE | WS_BORDER |
		ES_LEFT | ES_MULTILINE | ES_READONLY | ES_AUTOVSCROLL,
		MsgWarnX, MsgWarnY,
		MsgWarnW, MsgWarnH,
		ghWndMain, (HMENU)IDE_MSGWARN, hInstance, NULL);
	SendMessage(hWndMsgWarn, WM_SETFONT, (WPARAM)hFontTR12, MAKELPARAM(FALSE, 0));


	// Highlight this button (BS_DEFPUSHBUTTON), which is the default
	// among the shaking-measurement settings
	hWndPGVslctIn = CreateWindow("button", "Max. Speed (in/s)",
		WS_CHILD | WS_VISIBLE | WS_BORDER |
		BS_PUSHBUTTON | BS_TEXT | BS_CENTER | BS_VCENTER,
		PGVslctInX, PGVslctInY,
		PGVslctInW, PGVslctInH,
		ghWndMain, (HMENU)IDB_PGV_SELECT_IN, hInstance, NULL);
	SendMessage(hWndPGVslctIn, WM_SETFONT, (WPARAM)hFontTR12, MAKELPARAM(TRUE, 0));

	hWndPGVslctCm = CreateWindow("button", "(cm/s)",
		WS_CHILD | WS_VISIBLE | WS_BORDER |
		BS_PUSHBUTTON | BS_TEXT | BS_CENTER | BS_VCENTER,
		PGVslctCmX, PGVslctCmY,
		PGVslctCmW, PGVslctCmH,
		ghWndMain, (HMENU)IDB_PGV_SELECT_CM, hInstance, NULL);
	SendMessage(hWndPGVslctCm, WM_SETFONT, (WPARAM)hFontTR12, MAKELPARAM(TRUE, 0));

	// Minimalist error check
	if (	hWndLogo        == NULL ||
			hWndMetricEtc   == NULL ||
			hWndMaxima      == NULL ||
			hWndDateTime    == NULL ||
			hWndWhichOf     == NULL ||
			hWndOldest      == NULL ||
			hWndMostRecent  == NULL ||
			hWndBiggest     == NULL ||
			hWndPrev        == NULL ||
			hWndPrevX5      == NULL ||
			hWndNext        == NULL ||
			hWndNextX5      == NULL ||
			hWndSiteInfo    == NULL ||
			hWndQuit        == NULL ||
			hWndMsgWarn     == NULL ||
			hWndPGVslctIn   == NULL ||
			hWndPGVslctCm   == NULL)
		ErrExit("Some control item did\nnot initiate correctly.");
}	// MakeControls()


////////////////////////////////////////////////////////////////////////////
// Return TRUE if the two events are identical, FALSE otherwise.

// ***** process *****
Bool
MatchEvent(char * AA, char * BB)
{
	EqEntry		*EvtA;
	EqEntry		*EvtB;

	EvtA = (EqEntry *)AA;
	EvtB = (EqEntry *)BB;

	if (EvtA->Time != EvtB->Time)
		return(FALSE);

	if (!streq(EvtA->chTime, EvtB->chTime))
		return(FALSE);

	if (EvtA->PGAch1 != EvtB->PGAch1)
		return(FALSE);
	if (EvtA->PGAch2 != EvtB->PGAch2)
		return(FALSE);
	if (EvtA->PGAch3 != EvtB->PGAch3)
		return(FALSE);

	if (EvtA->PGVch1 != EvtB->PGVch1)
		return(FALSE);
	if (EvtA->PGVch2 != EvtB->PGVch2)
		return(FALSE);
	if (EvtA->PGVch3 != EvtB->PGVch3)
		return(FALSE);

	if (EvtA->Temperature != EvtB->Temperature)
		return(FALSE);

	return(TRUE);
}	// MatchEvent()


////////////////////////////////////////////////////////////////////////////
// Compare the size of the current earthquake to the one being sought,
// which is represented by the current value of MMImax.

// ***** process *****
BOOL
MatchSize()
{
	double	MMIlcl, lclPGAmax, lclPGVmax;

	lclPGAmax = MAX3(CurrentEq->PGAch1, CurrentEq->PGAch2, CurrentEq->PGAch3);
	lclPGVmax = MAX3(CurrentEq->PGVch1, CurrentEq->PGVch2, CurrentEq->PGVch3);
	MMIlcl = CurrentEq->MMI;
	
	if (MMIlcl == MMImax)
		return (TRUE);
	else
		return (FALSE);

}	// MatchSize()


////////////////////////////////////////////////////////////////////////////
// Given Instrumental Intensity, return standard ShakeMap phrases describing
// perceived shaking and potential damage.  Rounds and clips.

// ***** process *****
void
MMIphrases(double MMI, char * PercvdShake, char * PotDmg)
{
	int		MMIint;

	// Round, to make sense with range-centered MMI scale.
	MMIint = MAX(1, MIN(10, (int)(MMI + HALF)));

	switch (MMIint) {
	case 1:
		strcpy(PercvdShake, SM_SHAKE_PERCVD_I);
		strcpy(PotDmg, SM_POT_DMG_I);
		break;
	case 2:
	case 3:
		strcpy(PercvdShake, SM_SHAKE_PERCVD_II_III);
		strcpy(PotDmg, SM_POT_DMG_II_III);
		break;
	case 4:
		strcpy(PercvdShake, SM_SHAKE_PERCVD_IV);
		strcpy(PotDmg, SM_POT_DMG_IV);
		break;
	case 5:
		strcpy(PercvdShake, SM_SHAKE_PERCVD_V);
		strcpy(PotDmg, SM_POT_DMG_V);
		break;
	case 6:
		strcpy(PercvdShake, SM_SHAKE_PERCVD_VI);
		strcpy(PotDmg, SM_POT_DMG_VI);
		break;
	case 7:
		strcpy(PercvdShake, SM_SHAKE_PERCVD_VII);
		strcpy(PotDmg, SM_POT_DMG_VII);
		break;
	case 8:
		strcpy(PercvdShake, SM_SHAKE_PERCVD_VIII);
		strcpy(PotDmg, SM_POT_DMG_VIII);
		break;
	case 9:
		strcpy(PercvdShake, SM_SHAKE_PERCVD_IX);
		strcpy(PotDmg, SM_POT_DMG_IX);
		break;
	case 10:
		strcpy(PercvdShake, SM_SHAKE_PERCVD_X);
		strcpy(PotDmg, SM_POT_DMG_X);
		break;
	default:
		ErrExit("Programming error in MMIphrases().");
	}


}	// MMIphrases()


////////////////////////////////////////////////////////////////////////////
// Given overall dimensions of main window, compute all object locations.
//
// Rough layout:
//
// -------------------------------------------
// |                   |                 |   |
// |    USGS Logo      |                 |   |
// |                   |                 |   |
// | PGA,%g;PGV,",cm/s |                 |   |
// |  EQ Date & time   |                 |   |
// |(which of how many)|                 |   |
// |  Earliest Latest  |                 |   |
// |   B i g g e s t   |                 |   |
// |  Previous   Next  |                 | S |
// |  Station ID Info  |                 | c |
// |                   |     bargraph    | a |
// |-------------------|                 | l |
// |                   |                 | e |
// | messages/warnings |                 |   |
// |                   |                 |   |
// |-------------------|                 |   |
// |               Quit|                 |   |
// |                   |                 |   |
// | Peak Acceleration |                 |   |
// | PeakVelocity | cm |                 |   |
// -------------------------------------------

// ***** process *****
void
ObjectLayout()
{

	LONG	MinHeight, MinWidth;
	LONG	ControlCenter;

	// Leave room for bargraph at least 2.5 times the width of the scale
	// bar, and the scale bar and two equal-sized label bars.
	MinWidth = GUTTER_SIZE + CONTROL_WIDTH +
		GUTTER_SIZE + ((5 * ScaleWidth) / 2) +
		GUTTER_SIZE + ScaleWidth + (2 * LableWidth) + GUTTER_SIZE;
	// Leave room for at least logo, %g (doule high), and date-time windows.
	MinHeight = GUTTER_SIZE + LOGO_HEIGHT +
		GUTTER_SIZE + (2 * LineHeight) +
		GUTTER_SIZE + FatLineHeight + GUTTER_SIZE;

	// Force window size to be at least minimal for useful display
	MainWidth = MAX(MainWidth, MinWidth);
	MainHeight = MAX(MainHeight, MinHeight);

	// ----------------- Position everything else -----------------

	// "Control" is a mythical window encompasing all the controls
	// (but not the surrounding gutters), however many real objects
	// are sized and located relative to it.

	// Control objects:
	ControlX = GUTTER_SIZE;
	ControlY = GUTTER_SIZE;
	ControlW = CONTROL_WIDTH - GUTTER_SIZE - GUTTER_SIZE;
	ControlH = MainHeight - GUTTER_SIZE - GUTTER_SIZE;

	LogoX = ControlX + LOGO_WIDTH_OFF;
	LogoY = ControlY + LOGO_TOPGTTR;
	LogoW = LOGO_WIDTH;
	LogoH = LOGO_HEIGHT;


	MetricEtcX = ControlX;
	MetricEtcY = LogoY + LogoH + GUTTER_SIZE;
	MetricEtcW = ControlW;
	MetricEtcH = (6 * (6 * FatLineHeightTR18b)) / 7;

	MaximaX = ControlX;
	MaximaY = MetricEtcY + MetricEtcH + GUTTER_SIZE;
	MaximaW = ControlW;
	MaximaH = 4 * LineHeightTR18b;

	SiteInfoX = ControlX;
	SiteInfoY = MaximaY + MaximaH + (2 * GUTTER_SIZE);
	SiteInfoW = ControlW;
	SiteInfoH = LineHeightTR18b + FatLineHeightTR18b;


	DateTimeX = ControlX;
	DateTimeY = SiteInfoY + SiteInfoH + (4 * GUTTER_SIZE);
	DateTimeW = ControlW;
	DateTimeH = FatLineHeightTR18b;

	WhichOfX = ControlX;
	WhichOfY = DateTimeY + DateTimeH + GUTTER_SIZE;
	WhichOfW = ControlW;
	WhichOfH = FatLineHeightTR12b;

	// Double gutter between buttons is intended
	ControlCenter = ControlX + ControlW / 2;

	OldestX = ControlCenter - BUTTON_WIDTH - GUTTER_SIZE;
	OldestY = WhichOfY + WhichOfH + GUTTER_SIZE;
	OldestW = BUTTON_WIDTH;
	OldestH = FatLineHeightTR12b;

	MostRecentX = ControlCenter + GUTTER_SIZE;
	MostRecentY = OldestY;
	MostRecentW = BUTTON_WIDTH;
	MostRecentH = FatLineHeightTR12b;

	BiggestX = ControlCenter - BUTTON_WIDTH;
	BiggestY = OldestY + OldestH + GUTTER_SIZE;
	BiggestW = BUTTON_WIDTH * 2;
	BiggestH = FatLineHeightTR12b;


	PrevX = ControlX;
	PrevY = BiggestY + BiggestH + GUTTER_SIZE;
	PrevW = BUTTON_WIDTH;
	PrevH = FatLineHeightTR12b;

	PrevX5X = PrevX + PrevW + (GUTTER_SIZE / 2);
	PrevX5Y = PrevY;
	PrevX5W = CharWidthLcTR12b * 4;
	PrevX5H = PrevH;

	NextX5X = MostRecentX - (GUTTER_SIZE / 2);
	NextX5Y = PrevY;
	NextX5W = CharWidthLcTR12b * 4;
	NextX5H = PrevH;

	NextX = ControlX + ControlW - BUTTON_WIDTH;
	NextY = PrevY;
	NextW = BUTTON_WIDTH;
	NextH = PrevH;


	// Quit and message/warning
	MsgWarnX = ControlX;
	MsgWarnY = NextY + NextH + GUTTER_SIZE;
	MsgWarnW = ControlW;
	MsgWarnH = (2 * LineHeightTR12) + FatLineHeightTR12;

	QuitY = MsgWarnY + MsgWarnH + GUTTER_SIZE;
	QuitW = (2 * BUTTON_WIDTH) / 3;
	QuitH = FatLineHeightTR12;
	QuitX = ControlX + ControlW - QuitW;

	// (reference bottom up...)
	PGVcmButtomWidth = (8 * CharWidthLC);
	PGVslctInX = ControlX;
	PGVslctInY = ControlY + ControlH - FatLineHeight;
	PGVslctInW = ControlW - PGVcmButtomWidth - GUTTER_SIZE;
	PGVslctInH = FatLineHeight;

	PGVslctCmX = ControlX + PGVslctInW + GUTTER_SIZE;
	PGVslctCmY = PGVslctInY;
	PGVslctCmW = PGVcmButtomWidth;
	PGVslctCmH = PGVslctInH;

	// Scale and bargraph:

	IntTtlX = BarX;
	IntTtlY = ControlY;
	IntTtlH = (3 * FatLineHeightTR12b) / 2;
	IntTtlW = BarW + GUTTER_SIZE + ScaleW + PrcvdShkW + PotDmgW;

	ScaleX = MainWidth - ScaleWidth - (2 * LableWidth) - GUTTER_SIZE;
	ScaleY = ControlY + IntTtlH + GUTTER_SIZE;
	ScaleH = ControlH - (IntTtlH + GUTTER_SIZE);
	ScaleW = ScaleWidth;	// NEVER CHANGE THIS!

	PrcvdShkX = MainWidth - (2 * LableWidth) - GUTTER_SIZE;
	PrcvdShkY = ScaleY;
	PrcvdShkH = ScaleH;
	PrcvdShkW = LableWidth;
	
	PotDmgX = MainWidth - LableWidth - GUTTER_SIZE;
	PotDmgY = ScaleY;
	PotDmgH = ScaleH;
	PotDmgW = LableWidth;

	BarX = ControlX + ControlW + GUTTER_SIZE;
	BarY = ScaleY;
	BarH = ScaleH;
	BarW = MainWidth - ControlW - ScaleW - PrcvdShkW - PotDmgW - 4 * GUTTER_SIZE;

}	// ObjectLayout()


////////////////////////////////////////////////////////////////////////////
// Given the pair (PGA, PGV), return an instrumental intensity according to
// the ShakeMap equations.  Equivalent to ShakeMap routine get_mmi().
//
// If one or the other of PGA or PGV is zero or less, then the MMI will be
// based solely on the other value.  This feature is useful principaly for
// building the color scales.
//
// NOTE that the MMI values are constrained to fall between 1.0 and 10.0,
// and is ROUNDED to the nearest 0.01 units to obtain a more sensible
// display response in borederline cases.

// ***** process *****
double
PGAVtoMMI(double PGA, double PGV)		// cm/s/s and cm/s
{
	double		MMI, MMIpga, MMIpgv;	// Instrumental Intensity
	double		PGAweight, PGVweight;	// Ramp weighting between the MMIs

	double		PGAthresh = 65.96;		// cm/s/s equivalent to MMI==5.0
	double		PGVthresh = 5.809;		// cm/s   equivalent to MMI==5.0

	// Conversion coefficients (general form MMI = B + SM*log(PGx)):
	double		SMa    =  3.6598;		// PGA; MMI>5
	double		Ba     = -1.6582;
	double		SMaLow =  2.1987;		// PGA; MMI<5
	double		BaLow  =  1.0   ;

	double		SMv    =  3.4709;		// PGV; MMI>5
	double		Bv     =  2.3478;
	double		SMvLow =  2.0951;		// PGV; MMI<5
	double		BvLow  =  3.3991;

	// Compute Instrumental Intensities from each metric

	if (PGA >= PGAthresh) {
		MMIpga = Ba + SMa * log10(MAX(PGA, (double)0.1));
	}
	else {
		MMIpga = BaLow + SMaLow * log10(MAX(PGA, (double)0.1));
	}

	if (PGV >= PGVthresh) {
		MMIpgv = Bv + SMv * log10(MAX(PGV, (double)0.01));
	}
	else {
		MMIpgv = BvLow + SMvLow * log10(MAX(PGV, (double)0.01));
	}

	// As for ShakeMap, key on PGA's MMI, and between V and VII ramp
	// linearly between using PGA- and PGV-generated MMI (PGA for lower
	// MMIs and PGV for higher MMIs).  {But if one or the other is <= 0,
	// use the other entirely.}

	if (PGA > ZIP && PGV > ZIP) {
		if (MMIpga < FIVE)
			PGVweight = ZIP;
		else if (MMIpga > SEVEN)
			PGVweight = ONE;
		else {
			PGVweight = (MMIpga - FIVE) / TWO;
		}
		PGAweight = ONE - PGVweight;

		MMI = PGAweight * MMIpga  +  PGVweight * MMIpgv;
	}

	else if (PGV <= ZIP) {
		MMI = MMIpga;
	}

	else {
		MMI = MMIpgv;
	}


	// Clip between 1.0 and 10.0 and truncate precision at 0.01 units.
	MMI = floor(HALF + (MMI * HH)) / HH;
	MMI = MIN(MAX(MMI, ONE), 10.0);
	return MMI;

}	// PGAVtoMMI()


////////////////////////////////////////////////////////////////////////////
// Given an instrumental intensity, return a color (Red, Green, Blue) in
// the ShakeMap spectrum.

// ***** process *****
void
ShakeMapColor(SMCred, SMCgreen, SMCblue, MMI)
	int			*SMCred, *SMCgreen, *SMCblue;
	double		MMI;
{
	// { Red, Green, Blue } tripples, MMI = subscript.
	int			SMCS[14][3] = {
					{ 255, 255, 255 },
					{ 255, 255, 255 },	// Lowest MMI from PGAVtoMMI()
					{ 191, 204, 255 },
					{ 160, 230, 255 },
					{ 128, 255, 255 },
					{ 122, 255, 147 },
					{ 255, 255,   0 },
					{ 255, 200,   0 },
					{ 255, 145,   0 },
					{ 255,   0,   0 },
					{ 200,   0,   0 },	// Highest MMI from PGAVtoMMI()
					{ 176,   0,   0 },	// Added by JRE
					{ 152,   0,   0 },	// Added by JRE
					{ 128,   0,   0 },
				};
	int			lowMMI, highMMI;
	double		DlowMMI, DhighMMI;
	double		dRedL, dGreenL, dBlueL;
	double		dRedH, dGreenH, dBlueH;

	// Interpolate colors linearly between adjacent integer MMIs
	lowMMI  = (int)MMI;			DlowMMI  = (double)lowMMI;
	highMMI = (int)(MMI + ONE);	DhighMMI = (double)highMMI;

	dRedL   = (double)SMCS[lowMMI][0];
	dRedH   = (double)SMCS[highMMI][0];
	dGreenL = (double)SMCS[lowMMI][1];
	dGreenH = (double)SMCS[highMMI][1];
	dBlueL  = (double)SMCS[lowMMI][2];
	dBlueH  = (double)SMCS[highMMI][2];

	*SMCred = (int)( HALF + dRedL +
		(dRedH - dRedL) * (MMI - DlowMMI) / (DhighMMI - DlowMMI) );

	*SMCgreen = (int)( HALF + dGreenL +
		(dGreenH - dGreenL) * (MMI - DlowMMI) / (DhighMMI - DlowMMI) );

	*SMCblue = (int)( HALF + dBlueL +
		(dBlueH - dBlueL) * (MMI - DlowMMI) / (DhighMMI - DlowMMI) );

}	// ShakeMapColor()


////////////////////////////////////////////////////////////////////////////
// Set up COM1 for reading from TREMOR instrument.  This is a totally
// passive port that can only read incoming data and returns immediately
// whether any data are read or not.

// ***** process *****
HANDLE
StartCom1()
{
	HANDLE			Com1local;
	BOOL			success;
	DCB				dcb;
	COMMTIMEOUTS	timeouts;

	Com1local = CreateFile("COM1",
		GENERIC_READ,
		0, 0, OPEN_EXISTING,
		FILE_ATTRIBUTE_NORMAL, 0);

	if (Com1local == INVALID_HANDLE_VALUE) {
		sprintf(msgstr, "Could not open\nCOM1 (%d).", GetLastError());
		ErrExit(msgstr);
	}

	// Change settings of COM1 to the right ones
	success = GetCommState(Com1local, &dcb);
	if (!success) {
		sprintf(msgstr, "Could not get settings\nfor COM1 (%d).", GetLastError());
		ErrExit(msgstr);
	}

	// 9600, 8N1 (Raven/TREMOR standard settings)
	dcb.BaudRate		= 9600;
	dcb.ByteSize		= 8;
	dcb.Parity			= NOPARITY;
	dcb.StopBits		= ONESTOPBIT;

	dcb.fOutxCtsFlow	= FALSE;		// Disable RTS/CTS flow control
	dcb.fOutxDsrFlow	= FALSE;		// Disable DTR/DSR flow control
	dcb.fDsrSensitivity	= FALSE;		// Driver ignores DSR
	dcb.fOutX			= FALSE;		// Disable XON/XOFF output flow control
	dcb.fInX			= FALSE;		// Disable XON/XOFF input flow control
	dcb.fNull			= FALSE;		// Don't discard null bytes
	dcb.fAbortOnError	= FALSE;		// Don't abort rd/wr on error.

	success = SetCommState(Com1local, &dcb);
	if (!success) {
		sprintf(msgstr, "Could not change settings\nfor COM1 (%d).", GetLastError());
		ErrExit(msgstr);
	}

	// Make a large input buffer.  20000 is NT default.
   success = SetupComm(Com1local, (DWORD)COM1_BUFSIZE, (DWORD)0);
   if (!success)
		ErrExit("Could not change buffer size for COM1.");

	// Cause instant timeouts by ReadFile.
	timeouts.ReadIntervalTimeout = MAXDWORD;
	timeouts.ReadTotalTimeoutMultiplier = 0;
	timeouts.ReadTotalTimeoutConstant = 0;
	timeouts.WriteTotalTimeoutMultiplier = 0;
	timeouts.WriteTotalTimeoutConstant = 0;
	if (!SetCommTimeouts(Com1local, &timeouts)) {
		sprintf(msgstr, "Could not zero timeouts\nfor COM1 (%d).", GetLastError());
		ErrExit(msgstr);
	}

	// Clean up buffers
   success = PurgeComm(Com1local, PURGE_RXCLEAR);
   if (!success) {
		sprintf(msgstr, "Could not purge COM1\nbuffers (%d).", GetLastError());
		ErrExit(msgstr);
	}

	return(Com1local);
}	// StartCom1()


////////////////////////////////////////////////////////////////////////////
// Convert ADC counts to temperature for Ch 0.
//
// YSI 44031 thermistor using their test points at 0, 25, and 40 C from
// "Temperature Sensors and Probes, 1993, p. 11 and their algorithm on
// p. 74 (their BASIC program, but done on HP calculator).  (This is the
// "Steinhart-Hart thermistor equation", a widely accepted empiricle
// match to NTC thermistor behavior.)

// ***** process *****
double
Temperature(short int ADCcounts)
{
	double	temperature, resistance, voltage, lr;

	// From ADC count, compute voltage (and degain it).
	voltage = (double)ADCcounts * ADCtoV / (double)3.0;

	// From that compute resistance
	resistance = voltage * R0 / (Vref - voltage);

	// From that compute temperature
	lr = log(resistance);
	temperature = ONE / ( AAA + BBB * lr + DDD * lr * lr * lr );
	temperature = temperature - (double)273.15;	// K --> C

	return (temperature);
}	// Temperature()


////////////////////////////////////////////////////////////////////////////
// ADC vounts --> battery voltage

// ***** process *****
double
VBat(short int ADCcounts)
{
	return((double)ADCcounts * ADCtoV * (double)4.0);
}	// VBat()


////////////////////////////////////////////////////////////////////////////
// Compensate and expand (for human ease) any given text line.  Returns
// pointer to the expanded, compensated text line.  Input and output are
// NULL terminated strings with no CR or NL.
//
// If line is not recognized, NULL pointer is returned (the calling routine
// should back out gracefully).
//
// CAVEAT:  This version assumes that ONLY lines with ":TRG" are passed
//          to it.
//
// Conversions:
//     ":", ";", and " " --> "  "
//     epochal times --> "yyyy/mm/dd  hh:mm:ss.sss"
//     ACD counts --> degrees C, Volts, or (corrected) cm/s^2
//     Finish computations of Arias, PSV, and temperature function.
//     Translate some line ID codes (e.g., "END3") to English.
//
// CAVIAT:  GrabTRG() is very sensitive to the format
// returned by VerboseTRG().  Modify with great care.

// ***** process *****
char *
VerboseTRG(char * RawTextIn)
{
static	char	rslt[BUF_SIZE];
	char		RawText[BUF_SIZE];	// Buffer the input since mashing it
	char		*tokens[20];		// Array of pointers to parsed tokens
	char		*tokbuf;
	int			ntoks;				// Number of tokens found
static double	TmprtrA;			// C (STATIC for use with Arias and PSV)
	double		VoltsA;				// Battery voltage
	double		corwave[7];			// cm/s^2
	char		Cases[SHORT_STR];	// Line-type tag for switch()
	int			itk;
	char		TimeA[LONG_STR];	// For translating time
static double	trigger_time;
	double		PGA_time;

	BOOL		valid;				// TRUE when line IS expandable telemetry

	double		PGV1, PGV2, PGV3;
	double		Sa2, Sa3;
	char		SaTxt[6];

static VerSave		VSS;


	if (strlen(RawTextIn) > BUF_SIZE)
		ErrExit("Overflowed RawTextIn[]\nin Verbose().");

	// Don't mash original text
	strcpy(RawText, RawTextIn);

	///////////////////////////// PARSE LINE /////////////////////////////

	// When NOT expecting a waveform line, those lines containing a
	// ":" usually will be valid, expandable, general telemetry from the
	// instrument, and will need case-specific parsing.  So parse all the
	// tokens and then go to cases.
	if (NULL != strchr(RawText, (int)':')) {

		// First token
		tokens[0] = strtok(RawText, ":; ");
		ntoks = 1;
		tokens[1] = NULL;
		// Remaining tokens
		while (NULL != (tokbuf = strtok(NULL, ":; "))) {
			tokens[ntoks] = tokbuf;
			ntoks++;
		}

		for (itk = 0 ; itk < ntoks ; itk++) {
			if (strlen(tokens[itk]) > BUF_SIZE)
				ErrExit("Overflowed tokens[?]\nin Verbose().");
		}

		// Second token should be the line ID ...
		if (NULL != tokens[1])
			strcpy(Cases, tokens[1]);

		// ... and if there isn't one, bail out.  (Happens when editing
		// files via Internet, for example.)
		else {
			strcpy(rslt, RawTextIn);
			return(rslt);
		}
	}

	// Any unexpectedly colon-free line causes the default return
	else {
		strcpy(rslt, RawTextIn);
		return(rslt);
	}

	///////////////////////////// END PARSE //////////////////////////////


	//////////////////////// EXPAND AND CORRECT //////////////////////////

	// Unless otherwise discovered, the line is "valid" general telemetry
	valid = TRUE;

	// Trigger messages
	if (streq("TRG0", Cases)) {	// P3:TRG0;892752629.085;1372
		VERBOSE_ERR(4)

		VSS.RawTriggerTime = atof(tokens[2]);
		VSS.TrueTriggerTime = VSS.RawTriggerTime;	// Don't know any better

		VoltsA = VBat((short int)atoi(tokens[3]));
		strcpy(TimeA, EpochToStr(VSS.TrueTriggerTime));
		sprintf(rslt, "%s  TRIGGERED  (%.2lfV)%s%s ADC (Corrected by %.3lf s)",
			tokens[0], VoltsA, VerboseLB, TimeA, VSS.RawTriggerTime - VSS.TrueTriggerTime);
	}


	// -------PGA-------
	else if (streq("TRG1", Cases)) {	// P3:TRG1;6061;892752629.845;427
		VERBOSE_ERR(5)
		VSS.TmprtrA = Temperature((short int)atoi(tokens[4]));
		corwave[1] = Compensate(1, VSS.TmprtrA, (short int)atoi(tokens[2]));

		PGA_time = atof(tokens[3]);

		sprintf(rslt,
			"%s  Ch1 PGA=%7.2lf gal @ %.2lf s  %.2lfC",
			tokens[0],
			corwave[1],
			PGA_time - VSS.TrueTriggerTime,
			VSS.TmprtrA);
	}

	else if (streq("TRG2", Cases)) {	// P3:TRG2;10160;892752629.835;427
		VERBOSE_ERR(5)
		VSS.TmprtrA = Temperature((short int)atoi(tokens[4]));
		corwave[2] = Compensate(2, VSS.TmprtrA, (short int)atoi(tokens[2]));

		PGA_time = atof(tokens[3]);

		sprintf(rslt,
			"%s  Ch2 PGA=%7.2lf gal @ %.2lf s  %.2lfC",
			tokens[0],
			corwave[2],
			PGA_time - VSS.TrueTriggerTime,
			VSS.TmprtrA);
	}

	else if (streq("TRG3", Cases)) {	// P3:TRG3;878;892752629.115;427
		VERBOSE_ERR(5)
		VSS.TmprtrA = Temperature((short int)atoi(tokens[4]));
		corwave[3] = Compensate(3, VSS.TmprtrA, (short int)atoi(tokens[2]));

		PGA_time = atof(tokens[3]);

		sprintf(rslt,
			"%s  Ch3 PGA=%7.2lf gal @ %.2lf s  %.2lfC",
			tokens[0],
			corwave[3],
			PGA_time - VSS.TrueTriggerTime,
			VSS.TmprtrA);
	}
	// -------PGA-------


	// -------PGV-------
	// P13:TRGV;523 550;26080 504;40800 504
	else if (streq("TRGV", Cases)) {
		VERBOSE_ERR(8)

		// (assumes 100 sps in case BGN messages not seen during this run)
		// Uses temperature from PGA of channel 3 by default.
		PGV1 = CompPGV(atol(tokens[2]),
			Site.V_Sensitivity,
			Site.V_TCS, Site.V_Tref,
			VSS.TmprtrA, HH);

		PGV2 = CompPGV(atol(tokens[4]),
			Site.N_Sensitivity,
			Site.N_TCS, Site.N_Tref,
			VSS.TmprtrA, HH);

		PGV3 = CompPGV(atol(tokens[6]),
			Site.E_Sensitivity,
			Site.E_TCS, Site.E_Tref,
			VSS.TmprtrA, HH);

		sprintf(rslt,
			"%s  PGV Ch1=%6.2lf Ch2=%6.2lf Ch3=%6.2lf cm/s, %.2lfC (%.2lf, %.2lf, %.2lf s into)",
			tokens[0], PGV1, PGV2, PGV3, VSS.TmprtrA,
			atof(tokens[3]) / HH, atof(tokens[5]) / HH, atof(tokens[7]) / HH);
	}
	// -------PGV-------


	// -------Sa-------
	// P13:TRGS;300;429184 655921;6
	// P13:TRGS;1000;434058 696506;9
	// P13:TRGS;3000;492739 714771;12
	else if (streq("TRGS", Cases)) {
		VERBOSE_ERR(6)

		Sa2 = CompSa(atol(tokens[3]),
			Site.N_Sensitivity,
			Site.N_TCS, Site.N_Tref,
			VSS.TmprtrA, atof(tokens[5]));

		Sa3 = CompSa(atol(tokens[4]),
			Site.E_Sensitivity,
			Site.E_TCS, Site.E_Tref,
			VSS.TmprtrA, atof(tokens[5]));

		switch (atoi(tokens[2])) {
		case SaPer1:
			strcpy(SaTxt, SaNote1);
			break;
		case SaPer2:
			strcpy(SaTxt, SaNote2);
			break;
		case SaPer3:
			strcpy(SaTxt, SaNote3);
			break;
		default:
			strcpy(SaTxt, "Sa??");
		}

		sprintf(rslt,
			"%s  %s %4.2lf s  Ch2=%7.2lf  Ch3=%7.2lf cm/s",
			tokens[0], SaTxt, atof(tokens[2]) / KK, Sa2, Sa3);
	}
	// -------Sa-------


	// Default return for lines with a colon but no valid line ID
	else {
		strcpy(rslt, RawTextIn);
		valid = FALSE;
	}

	if (strlen(rslt) >= BUF_SIZE - 1)
		ErrExit("Verbose() result is too long.");

	return(rslt);
}	// VerboseTRG()


////////////////////////////////////////////////////////////////////////////
// Write out earthquake database (linked list) at close of session.

// ***** process *****
BOOL
WriteDataBase()
{
	EqEntry			*TempEq;				// Earthquake to output.

	// Move the event pointer to top of list
	ll_head(EqList);		// Null earthquake (head of list is always empty)

	fpHistoryFile = fopen(HistoryFile, "w");
	if (NULL == fpHistoryFile) {
		return(FALSE);
	}

	else {
		while (ll_next(EqList)) {

			TempEq = (EqEntry *)ll_ret(EqList);

			// Write out an earthquake (Sa will be %7.2lf)
			fprintf(fpHistoryFile,
				"%s  %7.3lf %7.3lf %7.3lf  %6.2lf %6.2lf %6.2lf  %6.2lf  %5.2lf\n",
				TempEq->chTime,
				TempEq->PGAch1, TempEq->PGAch2, TempEq->PGAch3,		// %g
				TempEq->PGVch1, TempEq->PGVch2, TempEq->PGVch3,		// cm/s
				TempEq->Temperature, TempEq->MMI);	// MMI is not read back in
		}
	
		fclose(fpHistoryFile);
		return(TRUE);
	}

}	// WriteDataBase()


////////////////////////////////////////////////////////////////////////////
// END

