// Bruce Julian's linked-list routines.
// Stolen 16 Nov 99 from andreas:/we/crhet/julian/src/lib/libq.

#include "local.h"
#include "lnklst.h"
#include "libq.h"

void		ErrExit(char *);

/*
 * Allocate storage for a link.
 */
// ***** process *****
PRIVATE LNKTYP *
llcrlink()
{
	LNKTYP	*link;

	link = (LNKTYP *)malloc(sizeof(LNKTYP));
	if (NULL == link)
		ErrExit("Memory allocation failed in llcrlink().");
	link->item = NULL;
	return link;
}

/*
 * Open a new (empty) list.
 */
// ***** process *****
LNKLST *
ll_open()
{
	LNKLST	*l;

	l = (LNKLST *)malloc(sizeof(LNKLST));
	if (NULL == l)
		ErrExit("Memory allocation failed in ll_open().");
	l->tail = l->clp = &l->head;
	l->head.prev = l->head.next = NULL;
	l->head.item = NULL;
	l->lstlen = 0;
	return l;
}

/*
 * Go to the next link.  Return FALSE if at tail of list, TRUE otherwise.
 */
// ***** process *****
Bool
ll_next(l)
	LNKLST	*l;
{
	if (l->clp->next == NULL)
		return FALSE;
	l->clp = l->clp->next;
	return TRUE;
}

/*
 * Go to the previous link.  Return FALSE if at head of list, TRUE otherwise.
 */
// ***** process *****
Bool
ll_prev(l)
	LNKLST	*l;
{
	if (l->clp->prev == NULL)
		return FALSE;
	l->clp = l->clp->prev;
	return (l->clp->prev != NULL);
}

/*
 * Search list for item matching *p .
 */
// ***** process *****
int
ll_find(l, p, fmatch)
	LNKLST	*l;
	char	*p;				// Item to match
	Bool	(*fmatch)();	// Comparison function
{
	while (ll_next(l))
		if ((*fmatch)(l->clp->item, p))
			return SUCCEED;
	return FAIL;
}

/*
 * Go to the head of list (which contains NO item).
 */
// ***** process *****
void
ll_head(l)
	LNKLST	*l;
{
	l->clp = &l->head;
}

/*
 * Go to the tail of list.
 */
// ***** process *****
void
ll_tail(l)
	LNKLST	*l;
{
	l->clp = l->tail;
}

/*
 * Retrieve the current item.
 */
// ***** process *****
char *
ll_ret(l)
	LNKLST	*l;
{
	return l->clp->item;
}

/*
 * Add new link after current one, go to it.
 */
// ***** process *****
void
ll_add(l, p)
	LNKLST	*l;
	char	*p;
{
	LNKTYP	*new;

	/* Create new link */
	new = llcrlink();
	new->item = p;

	/* Reset pointers */
	new->prev = l->clp;
	new->next = l->clp->next;
	if (l->clp == l->tail)
		l->tail = new;
	else
		l->clp->next->prev = new;
	l->clp->next = new;
	l->clp = new;

	l->lstlen++;
}

/*
 * Delete the current link, return its item, go to previous link.
 */
// ***** process *****
char *
ll_delete(l)
	LNKLST	*l;
{
	LNKTYP	*before, *after;
	char	*p;

	if (l->clp == &l->head)
		/* Can't delete head */
		return NULL;

	/* Reset the pointers */
	before = l->clp->prev;
	after = l->clp->next;
	before->next = after;
	if (after == NULL)
		/* At the tail */
		l->tail = before;
	else
		after->prev = before;

	/* Delete current link */
	p = l->clp->item;
	free((char *)(l->clp));
	--(l->lstlen);

	/* Go to previous link */
	l->clp = before;

	return p;
}

/*
 * Close a list, freeing storage.
 */
// ***** process *****
void
ll_close(l, fdel)
	LNKLST	*l;
	void	(*fdel)();	/* Item-deletion function		*/
{
	/* Delete all the links */
	ll_tail(l);
	while (ll_len(l) > 0)
		(*fdel)(ll_delete(l));

	/* Free the list structure */
	free((char *)l);
}
